@@ -12,18 +12,17 @@
 #ifndef __arm32_threads_h
 #define __arm32_threads_h
 
-/**/
-/* Thread handling */
-/**/
-#define	USE_INTERNAL_THREADS
-
-#define	THREADSTACKSIZE		(32 * 1024)
+#define	THREADSTACKSIZE		(64 * 1024)
 
 /*
  * Stack offset.
  * This is the offset into the setjmp buffer where the stack pointer is
  * stored.  This may be different with different OSes.
  */
+#if defined(__XSCALE__) || defined(HAVE_XSCALE)
+#define	SP_OFFSET		20
+#else
 #define	SP_OFFSET		23
+#endif /* defined(__XSCALE__) || defined(HAVE_XSCALE) */
 
 #endif
