@@ -20,12 +20,24 @@
 #include <features.h>
 #endif
 
+#include "i386/fpu-set.h"
+
 void
 init_md(void)
 {
 #if defined(M_MMAP_MAX) && defined(HAVE_MALLOPT)
 	mallopt(M_MMAP_MAX, 0);
 #endif
+
+	/* See http://www.srware.com/linux_numerics.txt */
+        /*
+	  This puts the X86 FPU in 64-bit precision mode.  The default
+	  under Linux is to use 80-bit mode, which produces subtle
+	  differences from FreeBSD and other systems, eg,
+	  (int)(1000*atof("0.3")) is 300 in 64-bit mode, 299 in 80-bit
+	  mode.
+        */
+	set_fpu_precision();
 }
 
 #if defined(__GLIBC__)
