@@ -17,17 +17,8 @@
 /**/
 #include "i386/jit.h"
 
-/**/
-/* Extra exception handling information. */
-/**/
-#include <siginfo.h>
-#include <sys/regset.h>		/* Bug in ucontext.h requires this */
-#include <signal.h>		/* Bug in ucontext.h requires this */
-#include <ucontext.h>
-
 /* Function prototype for signal handlers */
-#define	EXCEPTIONPROTO						\
-	int sig, siginfo_t* sip, ucontext_t* ctx
+#define	EXCEPTIONPROTO SIGNAL_ARGS(sig, ctx)
 
 /* Get the first exception frame from a signal handler */
 #define	EXCEPTIONFRAME(f, c)					\
