@@ -12,16 +12,11 @@
 #ifndef __parisc_hpux_threads_h
 #define __parisc_hpux_threads_h
 
-/* Stacks go up on the PARISC */
+/* Stacks grow up on the PARISC */
 #define	STACK_GROWS_UP	1
  
-/**/
-/* Thread handling */
-/**/ 
-#define USE_INTERNAL_THREADS  
-  
 /* Not sure on this... */
-#define THREADSTACKSIZE		(32 * 1024)
+#define THREADSTACKSIZE		(64 * 1024)
 
 /*
  * Stack offset.
