@@ -13,6 +13,7 @@
 #define __jsyscall_h
 
 #include "config.h"
+#include "config-int.h"
 
 #if defined(HAVE_NETDB_H)
 #include <netdb.h>
@@ -69,21 +70,22 @@ typedef struct SystemCallInterface {
 	int	(*_remove)(const char *);
 
 	int	(*_socket)(int, int, int, int *);
-	int	(*_connect)(int, struct sockaddr *, int, int timeout);
-	int	(*_bind)(int, struct sockaddr *, int);
+	int	(*_connect)(int, struct sockaddr *, socklen_t, int timeout);
+	int	(*_bind)(int, struct sockaddr *, socklen_t);
 	int	(*_listen)(int, int);
-	int	(*_accept)(int, struct sockaddr *, int*, int, int *);
+	int	(*_accept)(int, struct sockaddr *, socklen_t*, int, int *);
 	int	(*_sockread)(int, void*, size_t, int, ssize_t *);
-	int	(*_recvfrom)(int, void *, size_t, int, struct sockaddr *, int *,
+	int	(*_recvfrom)(int, void *, size_t, int, struct sockaddr *, socklen_t *,
 		int timeout, ssize_t *);
 	int	(*_sockwrite)(int, const void *, size_t, ssize_t *);
 	int	(*_sendto)(int, const void *, size_t, int, const struct sockaddr *,
 		int, ssize_t *);
 	int	(*_setsockopt)(int, int, int, const void *, int);
-	int	(*_getsockopt)(int, int, int, void *, int *);
-	int	(*_getsockname)(int, struct sockaddr *, int *);
-	int	(*_getpeername)(int, struct sockaddr *, int *);
+	int	(*_getsockopt)(int, int, int, void *, socklen_t *);
+	int	(*_getsockname)(int, struct sockaddr *, socklen_t *);
+	int	(*_getpeername)(int, struct sockaddr *, socklen_t *);
 	int	(*_sockclose)(int);
+        int     (*_sockShutdown)(int);
 	int	(*_gethostbyname)(const char *, struct hostent **);
 	int	(*_gethostbyaddr)(const char *, size_t, int, struct hostent **);
 
@@ -108,15 +110,104 @@ extern SystemCallInterface Kaffe_SystemCallInterface;
  * Define some convenience macros
  */
 
-#define	KOPEN(A,B,C,D)	(*Kaffe_SystemCallInterface._open)(A,B,C,D)
-#define	KREAD(A,B,C,D)	(*Kaffe_SystemCallInterface._read)(A,B,C,D)
-#define	KWRITE(A,B,C,D)	(*Kaffe_SystemCallInterface._write)(A,B,C,D)
-#define	KLSEEK(A,B,C,D)	(*Kaffe_SystemCallInterface._lseek)(A,B,C,D)
-#define	KCLOSE(A)	(*Kaffe_SystemCallInterface._close)(A)
-#define	KFSTAT(A,B)	(*Kaffe_SystemCallInterface._fstat)(A,B)
+/**
+ * Open a file in a platform-independant, thread-safe way, and
+ * set the filedescriptor to the opened file's descriptor.
+ *
+ * @param filename name of file to open.
+ * @param flags flags to pass to open
+ * @param permissions permissions with which the file is to be opened
+ * @param filedescriptor pointer to the filedescriptor to store the result in
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KOPEN(filename, flags, permissions, filedescriptor)	\
+  (*Kaffe_SystemCallInterface._open)(filename, flags, permissions, filedescriptor)
+
+/**
+ * Read bytes from a file into a buffer in a 
+ * platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to read from
+ * @param buffer buffer to read the bytes into
+ * @param length number of bytes to be read
+ * @param bytesRead number of bytes actually read
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KREAD(filedescriptor, buffer, length, bytesRead)	\
+  (*Kaffe_SystemCallInterface._read)(filedescriptor, buffer, length, bytesRead)
+
+/**
+ * Write bytes to a file from a buffer in a 
+ * platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to write to
+ * @param buffer buffer to write the bytes from
+ * @param length number of bytes to be written
+ * @param bytesWritten number of bytes actually written
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KWRITE(filedescriptor, buffer, length, bytesWritten)	\
+  (*Kaffe_SystemCallInterface._write)(filedescriptor, buffer, length, bytesWritten)
+
+/**
+ * Reposition read/write offset in a file in a 
+ * platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to stat
+ * @param offset offset to set
+ * @param whence how to set the offset
+ * @param new_offset new value of read/write offset
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KLSEEK(filedescriptor, offset, whence, new_offset) \
+  (*Kaffe_SystemCallInterface._lseek)(filedescriptor, offset, whence, new_offset)
+
+/**
+ * Close a file in a platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to close
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KCLOSE(filedescriptor)	\
+  (*Kaffe_SystemCallInterface._close)(filedescriptor)
+
+/**
+ * FStat a file in a platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to stat
+ * @param stats return buffer
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define	KFSTAT(filedescriptor, stats)	\
+  (*Kaffe_SystemCallInterface._fstat)(filedescriptor, stats)
 #define	KSTAT(A,B)	(*Kaffe_SystemCallInterface._stat)(A,B)
-#define KFTRUNCATE(A,B) (*Kaffe_SystemCallInterface._ftruncate)(A,B)
-#define KFSYNC(A)       (*Kaffe_SystemCallInterface._fsync)(A)
+
+/**
+ * FTruncate a file in a platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to truncate
+ * @param offset pffest at which the filedescriptor should be truncated
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define KFTRUNCATE(filedescriptor,offset) \
+  (*Kaffe_SystemCallInterface._ftruncate)(filedescriptor, offset)
+
+/**
+ * Synchrnoze a file in a platform-independant, thread-safe way.
+ *
+ * @param filedescriptor filedescriptor to synchronize
+ * 
+ * @return 0 on success, or errno on failure.
+ */
+#define KFSYNC(filedescriptor) \
+  (*Kaffe_SystemCallInterface._fsync)(filedescriptor)
 
 #define	KMKDIR(A,B)	(*Kaffe_SystemCallInterface._mkdir)(A,B)
 #define	KRMDIR(A)	(*Kaffe_SystemCallInterface._rmdir)(A)
@@ -169,6 +260,8 @@ extern SystemCallInterface Kaffe_SystemCallInterface;
 #define KPIPEREAD(A,B,C,D,E) (*Kaffe_SystemCallInterface._piperead)(A,B,C,D,E)
 #define KPIPEWRITE(A,B,C,D,E) (*Kaffe_SystemCallInterface._pipewrite)(A,B,C,D,E)
 
+#define KSOCKSHUTDOWN(A) (*Kaffe_SystemCallInterface._sockShutdown)(A)
+
 #define KAFFE_MMAP_READ 0
 #define KAFFE_MMAP_WRITE 1
 #define KAFFE_MMAP_PRIVATE 2
