@@ -2,9 +2,12 @@
  * native.h
  * Native method support.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
@@ -15,14 +18,14 @@
 #include "stddef.h"
 #include <jni.h>
 
-struct _methods;
+struct _jmethodID;
 struct Hjava_lang_Class;
 struct Hjava_lang_ClassLoader;
 struct _strconst;
 
 #if !defined(__DEFINED_METHOD)
 #define	__DEFINED_METHOD
-typedef struct _methods Method;
+typedef struct _jmethodID Method;
 #endif
 #if !defined(__DEFINED_UTF8CONST)
 #define	__DEFINED_UTF8CONST
