@@ -2,9 +2,12 @@
  * baseClasses.c
  * Handle base classes.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution
  * of this file.
  */
@@ -32,7 +35,6 @@
 #include "itypes.h"
 #include "gc.h"
 #include "md.h"
-#include "java_lang_Cloneable.h"
 #include "gcj/gcj.h"
 #include "xprofiler.h"
 #if defined(KAFFE_FEEDBACK)
@@ -42,6 +44,9 @@
 #include "fileSections.h"
 #include "verify-type.h"
 #include "jar.h"
+#if defined(ENABLE_BINRELOC)
+#include "binreloc.h"
+#endif
 
 Utf8Const* init_name;
 Utf8Const* final_name;
@@ -55,11 +60,12 @@ Utf8Const* Exceptions_name;
 Utf8Const* SourceFile_name;
 Utf8Const* InnerClasses_name;
 
-static Hjava_lang_Class dummyClassClass;
-static Hjava_lang_Class* ClassClass = &dummyClassClass;
+static struct _dispatchTable *ClassClass_vtable;
+static Hjava_lang_Class*	ClassClass;
 Hjava_lang_Class* StringClass;
 Hjava_lang_Class* ObjectClass;
 Hjava_lang_Class* SystemClass;
+Hjava_lang_Class* RuntimeClass;
 Hjava_lang_Class* SerialClass;
 Hjava_lang_Class* CloneClass;
 Hjava_lang_Class* PtrClass;
@@ -79,7 +85,14 @@ Hjava_lang_Class* javaLangIntegerClass;
 Hjava_lang_Class* javaLangLongClass;
 Hjava_lang_Class* javaLangFloatClass;
 Hjava_lang_Class* javaLangDoubleClass;
-Hjava_lang_Class* kaffeLangAppClassLoaderClass;
+
+Hjava_lang_Class *javaNioBufferClass;
+Hjava_lang_Class *javaNioDirectByteBufferImplClass;
+Hjava_lang_Class *javaNioDirectByteBufferImplReadWriteClass;
+Hjava_lang_Class *gnuClasspathPointerClass;
+
+Field *gnuClasspathPointerAddress;
+Field *directByteBufferImplAddress;
 
 Hjava_lang_Class* javaLangThrowable;
 Hjava_lang_Class* javaLangVMThrowable;
@@ -94,14 +107,27 @@ Hjava_lang_Class* javaLangStackOverflowError;
 /* Let's not load this if we can't open Klasses.jar */
 Hjava_lang_Class* javaIoIOException;
 
-Hjava_lang_ClassLoader* appClassLoader;
-
-#define SYSTEMCLASS "java/lang/System"
-#define	SERIALCLASS "java/io/Serializable"
-#define	CLONECLASS  "java/lang/Cloneable"
-#define	LOADERCLASS "java/lang/ClassLoader"
-#define PTRCLASS    "kaffe/util/Ptr"
+static Hjava_lang_Class** stateCompleteClass[] = {
+  &StringClass,
+  &javaLangThrowable,
+  &javaLangVMThrowable,
+  &javaLangStackTraceElement,
+  &javaLangException,
+  &javaLangArrayIndexOutOfBoundsException,
+  &javaLangNullPointerException,
+  &javaLangArithmeticException,
+  &javaLangClassNotFoundException,
+  &javaLangNoClassDefFoundError,
+  &javaLangStackOverflowError,
+  NULL
+};
 
+#define RUNTIMECLASS "java/lang/Runtime"
+#define SYSTEMCLASS  "java/lang/System"
+#define	SERIALCLASS  "java/io/Serializable"
+#define	CLONECLASS   "java/lang/Cloneable"
+#define	LOADERCLASS  "java/lang/ClassLoader"
+#define PTRCLASS     "org/kaffe/util/Ptr"
 
 /* Initialisation prototypes */
 void initClasspath(void);
@@ -111,14 +137,10 @@ void initNative(void);
  * get the java.lang.Class class.
  * Load it if it is not loaded.
  */
-Hjava_lang_Class* 
-getClassClass(void)
+struct _dispatchTable*
+getClassVtable(void)
 {
-  if (ClassClass == NULL) {
-    loadStaticClass(&ClassClass, CLASSCLASS);
-  }
-
-  return ClassClass;
+  return ClassClass_vtable;
 }
 
 /**
@@ -151,6 +173,10 @@ initialiseKaffe(void)
 	INIT_MD();
 #endif
 
+#if defined(ENABLE_BINRELOC)
+	br_init_lib(NULL);
+#endif
+
 	/* Register allocation types with gc subsystem */
 	main_collector = initCollector();
 	KGC_init(main_collector);
@@ -184,6 +210,7 @@ initialiseKaffe(void)
 	/* Initialise the (native) threading system */
 	initNativeThreads(threadStackSize);
 
+	initLocking();
 	initEngine();
 	KaffeVM_initClassPool();
 	KaffeVM_initJarCache();
@@ -234,7 +261,7 @@ initialiseKaffe(void)
 	      LocalVariableTable_name && ConstantValue_name &&
 	      Exceptions_name && SourceFile_name && InnerClasses_name)) {
 		DBG(INIT, dprintf("not enough memory to run kaffe\n"); );
-		ABORT();
+		KAFFEVM_ABORT();
 	}
 
 	DBG(INIT, dprintf("done\n"); );
@@ -258,8 +285,8 @@ initialiseKaffe(void)
 	/* Setup exceptions */
 	initExceptions();
 
-	/* Setup locking */
-	initLocking();
+	/* Init stuff for the java security model */
+	initialiseSecurity();
 
 	/* Init thread support */
 	initThreads();
@@ -285,10 +312,9 @@ abortWithEarlyClassFailure(errorInfo* einfo)
 		einfo->type, einfo->classname, einfo->mess);
 	);
 
-	EXIT(-1);
+	KAFFEVM_EXIT(-1);
 }
 
-
 /*
  * We need to use certain classes in the internal machine so we better
  * get them in now in a known way so we can refer back to them.
@@ -296,30 +322,32 @@ abortWithEarlyClassFailure(errorInfo* einfo)
 void
 initBaseClasses(void)
 {
-        errorInfo einfo;
+	errorInfo einfo;
+	int i;
 
-	memset(&dummyClassClass, 0, sizeof(dummyClassClass));
+	DBG(INIT, dprintf("initBaseClasses()\n"); );
 
-        /* Primitive types */
-        initTypes();
+	/* Primitive types */
+	initTypes();
 	initVerifierPrimTypes();
 
-        DBG(INIT, dprintf("initBaseClasses()\n"); );
+	loadStaticClass(&ObjectClass, OBJECTCLASS);
+	loadStaticClass(&SerialClass, SERIALCLASS);
+	loadStaticClass(&CloneClass,  CLONECLASS);
+	loadStaticClass(&ClassClass,  CLASSCLASS);
+
+	ClassClass_vtable = ClassClass->vtable;
+	ObjectClass->head.vtable = ClassClass_vtable;
+	SerialClass->head.vtable = ClassClass_vtable;
+	CloneClass->head.vtable  = ClassClass_vtable;
+	ClassClass->head.vtable  = ClassClass_vtable;
+
+	finishTypes();
+
+	loadStaticClass(&StringClass, STRINGCLASS);
+	loadStaticClass(&SystemClass, SYSTEMCLASS);
+        loadStaticClass(&RuntimeClass, RUNTIMECLASS);
 
-        /* The base types */
-        loadStaticClass(&ObjectClass, OBJECTCLASS);
-        loadStaticClass(&SerialClass, SERIALCLASS);
-        loadStaticClass(&CloneClass, CLONECLASS);
-        loadStaticClass(&ClassClass, CLASSCLASS);
-        loadStaticClass(&StringClass, STRINGCLASS);
-        loadStaticClass(&SystemClass, SYSTEMCLASS);
-	
-        /* We must to a little cross tidying */
-        ObjectClass->head.vtable = ClassClass->vtable;
-        SerialClass->head.vtable = ClassClass->vtable;
-        CloneClass->head.vtable = ClassClass->vtable;
-        ClassClass->head.vtable = ClassClass->vtable;
-	
 	/* Basic types classes */
 	loadStaticClass(&javaLangVoidClass, "java/lang/Void");
 	loadStaticClass(&javaLangBooleanClass, "java/lang/Boolean");
@@ -332,7 +360,6 @@ initBaseClasses(void)
 	loadStaticClass(&javaLangDoubleClass, "java/lang/Double");
 	loadStaticClass(&PtrClass, PTRCLASS);
 	loadStaticClass(&ClassLoaderClass, LOADERCLASS);
-	loadStaticClass(&kaffeLangAppClassLoaderClass, APPCLASSLOADERCLASS);
 
 	/* Exception handling types */
 	loadStaticClass(&javaLangThrowable, "java/lang/Throwable");
@@ -353,14 +380,34 @@ initBaseClasses(void)
 	loadStaticClass(&javaLangRefSoftReference, "java/lang/ref/SoftReference");
 	loadStaticClass(&javaLangRefPhantomReference, "java/lang/ref/PhantomReference");
 
+	/* NIO helpers */
+	loadStaticClass(&javaNioBufferClass, "java/nio/Buffer");
+	loadStaticClass(&javaNioDirectByteBufferImplClass, "java/nio/DirectByteBufferImpl");
+	loadStaticClass(&javaNioDirectByteBufferImplReadWriteClass, "java/nio/DirectByteBufferImpl$ReadWrite");
+#if SIZEOF_VOID_P == 4
+	loadStaticClass(&gnuClasspathPointerClass, "gnu/classpath/Pointer32");
+#elif SIZEOF_VOID_P == 8
+	loadStaticClass(&gnuClasspathPointerClass, "gnu/classpath/Pointer64");
+#else
+#error "Unknown pointer size"
+#endif
+
 	DBG(INIT, dprintf("initBaseClasses() done\n"); );
 
-	/* Fixup primitive types */
-	finishTypes();
-	
-	if (!processClass(StringClass, CSTATE_COMPLETE, &einfo))
-		abortWithEarlyClassFailure(&einfo);
+	for (i = 0; stateCompleteClass[i] != NULL; i++) {
+	    if (!processClass(*stateCompleteClass[i], CSTATE_COMPLETE, &einfo))
+	      abortWithEarlyClassFailure(&einfo);
+	}	
 
-	appClassLoader = NULL;
+	/* Preresolve some fields which will be used in JNI. */	
+	gnuClasspathPointerAddress = KNI_lookupFieldC(gnuClasspathPointerClass, "data", false, &einfo);
+	directByteBufferImplAddress = KNI_lookupFieldC(javaNioDirectByteBufferImplClass, "address", false, &einfo);
+
+	if (gnuClasspathPointerAddress == NULL ||
+	    directByteBufferImplAddress == NULL)
+	  {
+	    DBG(INIT, dprintf("Cannot resolve fields necessary for NIO operations\n"); );
+	    KAFFEVM_EXIT(-1);
+	  }
 }
 
