@@ -84,8 +84,16 @@ typedef enum {
 struct Hjava_lang_Class {
 	Hjava_lang_Object	head;		/* A class is an object too */
 
+	/* these fields are also accessed from java and therefore MUST NOT BE CHANGED */
+
+	HArrayOfObject*			signers;
+	struct Hjava_security_ProtectionDomain*	protectionDomain;
+	struct Hjava_lang_Object*	vmData; /* unused for kaffe */
+	struct Hjava_lang_Object*	constructor;
+
+	/* end of fields being accessed from java */
+
 	struct _iLock*		lock;		/* Lock for internal use */
-        void *finalizer_call;
 
 	/* Link to class entry */
 	struct _classEntry*	centry;
@@ -110,7 +118,7 @@ struct Hjava_lang_Class {
 
 	/* Number of methods in the dtable. */
 	/* If CLASS_IS_PRIMITIVE, then the CLASS_PRIM_SIG. */
-	short			msize;
+	unsigned short			msize;
 
 	/* Pointer to array of Fields, on for each field.
 	   Static fields come first. */
@@ -133,7 +141,7 @@ struct Hjava_lang_Class {
 	short*			if2itable;	/* redundant now */
 	void**			itable2dtable;
 	short			interface_len;
-	short			total_interface_len;
+	unsigned short		total_interface_len;
 	/* pointers to all itable2dtable entries of classes implementing this interface */
 	void***			implementors;	/* interfaces only */
         int			impl_index;    /* index of the class in the implementors array. */
@@ -171,12 +179,6 @@ struct Hjava_lang_Class {
 	/** Stab type ID. */
 	int stab_id;
 #endif
-
-	/** The array of 'signer' objects; usually Certificates. */
-	HArrayOfObject*		signers;
-
-	/** The protection domain */
-	struct Hjava_security_ProtectionDomain*	protectionDomain;
 };
 
 #ifndef __DEFINED_CLASS
@@ -294,13 +296,14 @@ typedef struct _innerClass {
 typedef struct _parsed_signature {
 	Utf8Const*		signature;
 	u2			nargs;
+	u2			real_nargs;
 	u2			ret_and_args[1]; /* index into signature */
 	/* ret_and_args[0]: return value
 	   ret_and_args[1]: first argument
 	   etc */
 } parsed_signature_t;
 
-typedef struct _methods {
+typedef struct _jmethodID {
 	Utf8Const*		name;
 	parsed_signature_t*	parsed_sig;
 	accessFlags		accflags;
@@ -326,7 +329,7 @@ typedef struct _methods {
 	int			ndeclared_exceptions;
 	union {
 		constIndex*		local_exceptions;
-		struct _methods*	remote_exceptions;
+		struct _jmethodID*	remote_exceptions;
 	} declared_exceptions_u;
 #define declared_exceptions declared_exceptions_u.local_exceptions
 	int			framesize;	/* JIT only: size of frame */
@@ -345,8 +348,9 @@ typedef struct _methods {
 #define PSIG_UTF8(sig)		((sig)->signature)
 #define PSIG_DATA(sig)		(PSIG_UTF8((sig))->data)
 #define PSIG_RET(sig)		((sig)->ret_and_args[0])
-#define PSIG_NARGS(sig)		(sig->nargs)
+#define PSIG_NARGS(sig)		((sig)->nargs)
 #define PSIG_ARG(sig,n)		((sig)->ret_and_args[1+n])
+#define PSIG_RNARGS(sig)	((sig)->real_nargs)
 
 #define METHOD_PSIG(M)		((M)->parsed_sig)
 #define METHOD_SIG(M)		(PSIG_UTF8(METHOD_PSIG((M))))
@@ -368,7 +372,7 @@ typedef struct _dispatchTable {
 #define	DTABLE_METHODOFFSET	(sizeof(void*)*2)
 #define	DTABLE_METHODSIZE	(sizeof(void*))
 
-typedef struct _fields {
+typedef struct _jfieldID {
 	Hjava_lang_Class*	clazz;
 	Utf8Const*		name;
 	Utf8Const*		signature;
@@ -507,18 +511,61 @@ struct classFile;
 #define CLASS_CONST_USHORT2(CL, INDEX) \
   ((uint16)((CL)->constants.data[INDEX] >> 16))
 
-
-/*
+/**
  * 'processClass' is the core of the class initialiser and can prepare a
  * class from the cradle to the grave.
+ *
+ * @param clazz The class to process.
+ * @param state The state to achieve during this process.
+ * @param einfo An error which may have occured.
+ *
+ * @returns false if an error has occured, true if everything is ok.
  */
-bool			processClass(Hjava_lang_Class*, int, errorInfo *einfo);
+bool			processClass(Hjava_lang_Class* clazz, int state, errorInfo *einfo);
 
-Hjava_lang_Class*	loadClass(Utf8Const*, Hjava_lang_ClassLoader*, errorInfo *einfo);
-Hjava_lang_Class*	loadArray(Utf8Const*, Hjava_lang_ClassLoader*, errorInfo *einfo);
-Hjava_lang_Class* 	findClass(struct _classEntry* centry, errorInfo *einfo);
+/**
+ * This function loads a class using the given class loader if non-NULL else it uses the 
+ * internal bootstrap class loader to fetch classes. The classes are processed to the linked state.
+ * This function must not be used to "load" a class representing array. For that use loadArray.
+ * 
+ * @param name The name of the class to load.
+ * @param loader The loader to use. It may be NULL if you want to use the internal VM class loader.
+ * @param einfo The error structure to fill if something bad happens.
+ *
+ * @returns A valid class object if the load is successful, NULL otherwise.
+ */
+Hjava_lang_Class*	loadClass(Utf8Const* name, Hjava_lang_ClassLoader* loader, errorInfo *einfo);
 
-void			loadStaticClass(Hjava_lang_Class**, const char*);
+/**
+ * This function loads an array with the given name and using the given class loader. The array is usable directly
+ * after that call.
+ *
+ * @param name The name of the array to load.
+ * @param loader The loader to use. It may be NULL If you want to use the internal VM class loader.
+ * @param einfo The error structure to fill if something bad happens.
+ *
+ * @returns A valid class object if the load is successful, NULL otherwise.
+ */
+Hjava_lang_Class*	loadArray(Utf8Const* name, Hjava_lang_ClassLoader* loader, errorInfo *einfo);
+
+/**
+ * This function finds a class in a directory or in a jar file. It implements the internal VM class loader.
+ *
+ * @param centry The class entry representing the class to be loaded.
+ * @param einfo The error structure to fill if something bad happens.
+ *
+ * @returns A valid class object if the load is successful, NULL otherwise.
+ */
+Hjava_lang_Class*	findClass(struct _classEntry* centry, errorInfo *einfo);
+
+/**
+ * This function is used at the VM boot to preload some classes. The classes are loaded and put in the linked state.
+ * It cannot return an error. If an error happened the VM is aborted.
+ *
+ * @param clazz A pointer to the place where the class object should be put.
+ * @param name The name of the class to load.
+ */
+void			loadStaticClass(Hjava_lang_Class** clazz, const char* name);
 
 Hjava_lang_Class*	setupClass(Hjava_lang_Class*, constIndex,
 				   constIndex, u2, Hjava_lang_ClassLoader*, errorInfo*);
@@ -543,8 +590,6 @@ classEntry*		lookupClassEntryInternal(Utf8Const*,
 int			removeClassEntries(Hjava_lang_ClassLoader*);
 void 			walkClassEntries(Collector *collector, void *gc_info, Hjava_lang_ClassLoader*);
 
-Collector* 		initCollector(void);
-
 Hjava_lang_Class*	lookupClass(const char*, Hjava_lang_ClassLoader*,
 				errorInfo*);
 Hjava_lang_Class*	lookupArray(Hjava_lang_Class*, errorInfo*);
@@ -561,6 +606,7 @@ void			establishMethod(Method*);
 Hjava_lang_Class*	getClassFromSignature(const char*, Hjava_lang_ClassLoader*, errorInfo*);
 Hjava_lang_Class*	getClassFromSignaturePart(const char*, Hjava_lang_ClassLoader*, errorInfo*);
 int			countArgsInSignature(const char *);
+int			KaffeVM_countRealNumberOfArgs(parsed_signature_t *);
 parsed_signature_t*	parseSignature(Utf8Const *, errorInfo*);
 
 int			startFields(Hjava_lang_Class*, u2 fieldct, errorInfo*);
@@ -570,34 +616,38 @@ void			destroyClassLoader(Collector *, void *);
 struct Hjava_lang_String* resolveString(Hjava_lang_Class* clazz, int idx,
 					errorInfo *einfo);
 int			findPackageLength(const char *name);
-/*
+
+/**
  * Start a search for a class.  If no other thread is searching for this
  * mapping then the responsibility falls on the current thread.
  *
- * ce - The mapping to start searching for.
- * out_cl - A placeholder for the class if it has already been bound.
- * einfo - An uninitialized errorInfo object.
- * returns - True, if the class is already bound or if this thread should be
+ * @param ce The mapping to start searching for.
+ * @param out_cl A placeholder for the class if it has already been bound.
+ * @param einfo An uninitialized errorInfo object.
+ *
+ * @returns True, if the class is already bound or if this thread should be
  *   responsible for searching/loading the class.  False, if searching for this
  *   class would result in a class circularity.
  */
 int classMappingSearch(classEntry *ce,
 		       Hjava_lang_Class **out_cl,
 		       errorInfo *einfo);
-/*
+/**
  * Start loading a class.
  *
- * ce - The mapping to start searching for.
- * out_cl - A placeholder for the class if it has already been bound.
- * einfo - An uninitialized errorInfo object.
- * returns - True, if the class is already bound or if this thread should be
+ * @param ce The mapping to start searching for.
+ * @param out_cl A placeholder for the class if it has already been bound.
+ * @param einfo An uninitialized errorInfo object.
+ *
+ * @returns True, if the class is already bound or if this thread should be
  *   responsible for searching/loading the class.  False, if searching for this
  *   class would result in a class circularity.
  */
 int classMappingLoad(classEntry *ce,
 		     Hjava_lang_Class **out_cl,
 		     errorInfo *einfo);
-/*
+
+/**
  * Transition a mapping to the loaded state.
  *
  * ce - The name mapping whose state should be updated.
@@ -606,7 +656,8 @@ int classMappingLoad(classEntry *ce,
  *   it will be the value previously stored in the mapping.
  */
 Hjava_lang_Class *classMappingLoaded(classEntry *ce, Hjava_lang_Class *cl);
-/*
+
+/**
  * Force a mapping to a particular state.
  *
  * ce - The name mapping whose state should be updated.
@@ -614,8 +665,19 @@ Hjava_lang_Class *classMappingLoaded(classEntry *ce, Hjava_lang_Class *cl);
  */
 void setClassMappingState(classEntry *ce, name_mapping_state_t nms);
 
+/**
+ * This function walks through the class pool and execute the given function
+ * for each class of the pool.
+ *
+ * @param walker A class walker.
+ * @param param A user parameter.
+ */
 void walkClassPool(int (*walker)(Hjava_lang_Class *clazz, void *), void *param);
 
+
+/**
+ * Initialize internal global variables of the class pool.
+ */
 void KaffeVM_initClassPool(void);
 
 extern Utf8Const* init_name;		/* "<clinit>" */
@@ -630,4 +692,6 @@ extern Utf8Const* Exceptions_name;	/* "Exceptions" */
 extern Utf8Const* SourceFile_name;	/* "SourceFile" */
 extern Utf8Const* InnerClasses_name;	/* "InnerClasses" */
 
+void initialiseSecurity (void);
+
 #endif
