@@ -345,7 +345,14 @@ int classMappingSearch(classEntry *ce,
 		case NMS_SEARCHING:
 			if( ce->data.thread == jt )
 			{
+				/* Circularity. */
 				done = 1;
+				retval = 0;
+				postExceptionMessage(
+					einfo,
+					JAVA_LANG(ClassNotFoundException),
+					"%s",
+					ce->name->data);
 				break;
 			}
 			waitStaticCond(&ce->slock, (jlong)0);
@@ -387,6 +394,8 @@ int classMappingSearch(classEntry *ce,
 			*out_cl = ce->data.cl;
 			done = 1;
 			break;
+		default:
+			break;
 		}
 		unlockStaticMutex(&ce->slock);
 	}
@@ -452,6 +461,8 @@ int classMappingLoad(classEntry *ce,
 			*out_cl = ce->data.cl;
 			done = 1;
 			break;
+		default:
+			break;
 		}
 		unlockStaticMutex(&ce->slock);
 	}
@@ -483,6 +494,9 @@ Hjava_lang_Class *classMappingLoaded(classEntry *ce, Hjava_lang_Class *cl)
 			ce->data.cl = cl;
 			retval = cl;
 			break;
+		case NMS_EMPTY:
+		case NMS_LOADED:
+		case NMS_DONE:
 		default:
 			/* Ignore. */
 			retval = ce->data.cl;
@@ -546,7 +560,7 @@ walkClassPool(int (*walker)(Hjava_lang_Class *clazz, void *), void *param)
 	for (ipool = CLASSHASHSZ;  --ipool >= 0; ) {
 		entry = classEntryPool[ipool];
 		for (; entry != NULL; entry = entry->next) {
-			if (entry->data.cl
+			if (entry->state != NMS_EMPTY && entry->data.cl
 			    && entry->loader == entry->data.cl->loader) {
 				walker(entry->data.cl, param);
 			}
