@@ -26,13 +26,13 @@
 
 #include <stdio.h>
 
-#include <errno.h>
+#include "lerrno.h"
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
 #include "config-io.h"
 #include "config-signal.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "gtypes.h"
 #include "gc.h"
 #include "debug.h"
