@@ -34,6 +34,7 @@
 #include "object.h"
 #include "constants.h"
 #include "md.h"
+#include "methodcalls.h"
 #include "callKaffeException.h"
 #include "classMethod.h"
 #include "code.h"
@@ -67,10 +68,6 @@ vmExcept_getSyncObj(VmExceptHandler* eh)
 }
 
 #define FRAMEOBJECT(O, F, E)    (O) = vmExcept_getSyncObj((VmExceptHandler*)(F))
-
-
-#define DISPATCH_EXCEPTION(F, H, E) vmExcept_setPC((VmExceptHandler *)(F), (H));  \
-                                    vmExcept_jumpToHandler((VmExceptHandler *)(F)); /* Does not return */
 #else
 
 #define DISPATCH_EXCEPTION(F,H,E) thread_data->exceptObj = NULL;\
@@ -112,7 +109,7 @@ vmExcept_setJNIFrame(VmExceptHandler* eh, JNIFrameAddress fp)
 	eh->frame.jni.fp = fp;
 }
 
-static void
+void
 vmExcept_jumpToHandler(VmExceptHandler* frame)
 {
 	JTHREAD_LONGJMP(frame->jbuf, 1);
@@ -457,7 +454,7 @@ dispatchException(Hjava_lang_Throwable* eobj, stackTraceInfo* baseFrame)
 		/* If handler found, call it */
 		if (foundHandler) {
 			thread_data->needOnStack = STACK_HIGH;
-			DISPATCH_EXCEPTION(frame->fp, handler, eobj); /* doesn't return */
+			engine_dispatchException(frame->fp, handler, eobj); /* doesn't return */
 		}
 
 #if defined(ENABLE_JVMPI)
