@@ -12,13 +12,14 @@
  * of this file. 
  */
 
+#include "config.h"
+
 #if defined(HAVE_STDARG_H)
 #include <stdarg.h>
 #endif /* defined(HAVE_STDARG_H) */
 
 #include <stdio.h>
 
-#include "config.h"
 #include "debug.h"
 #include "config-std.h"
 #include "config-mem.h"
@@ -46,7 +47,7 @@
 #endif
 #include "jni_i.h"
 #ifdef ENABLE_BINRELOC
-#include "prefix.h"
+#include "binreloc.h"
 #endif
 
 #ifdef __riscos__
@@ -77,27 +78,16 @@ static char *libraryPath = NULL;
 
 #ifdef ENABLE_BINRELOC
 static
-const char *discoverLibraryHome(void)
+char *discoverLibraryHome(void)
 {
-  static char libraryHome[MAXPATHLEN];
   char *entryPoint;
 
-  strcpy(libraryHome, SELFPATH);
-  
-  if (strlen(file_separator) != 1)
-    {
-      fprintf(stderr, "WARNING: Impossible to auto-discover the home of native libraries\n");
-      return NULL;
-    }
-
-  entryPoint = strrchr(libraryHome, file_separator[0]);
+  entryPoint = br_find_exe_dir(NULL);
   if (entryPoint == NULL)
     // Abnormal. We may return ".".
     return ".";
 
-  *entryPoint = 0;
-  
-  return libraryHome;
+  return entryPoint;
 }
 #endif
 
@@ -183,11 +173,11 @@ initNative(void)
 			return;
 		}
 	}
-	dprintf("Failed to locate native library \"%s\" in path:\n", NATIVELIBRARY);
+	dprintf("Failed to locate native library \"%s\" in path:\n", lib);
 	dprintf("\t%s\n", libraryPath);
 	dprintf("Aborting.\n");
 	fflush(stderr);
-	EXIT(1);
+	KAFFEVM_EXIT(1);
 }
 
 /*
@@ -196,7 +186,7 @@ initNative(void)
  * fills errbuf (if not NULL) with the error message. Assumes synchronization.
  */
 int
-loadNativeLibrary(char* path, struct Hjava_lang_ClassLoader* loader, char *errbuf, size_t errsiz)
+loadNativeLibrary(const char* path, struct Hjava_lang_ClassLoader* loader, char *errbuf, size_t errsiz)
 {
 	struct _libHandle *lib;
 	int libIndex;
