@@ -10,16 +10,26 @@
  */
 
 /* Use the incremental garbage collector */
-#include "mem/gc-incremental.c"
+#include "config-std.h"
+#include "gc.h"
+#include "kaffe/jmalloc.h"
+
+Collector* main_collector;
 
 void*
-__malloc(size_t sz)
+jmalloc(size_t sz)
 {
-	return ((*Kaffe_GarbageCollectorInterface._malloc)(sz, GC_ALLOC_FIXED));
+	return (KGC_malloc(main_collector, sz, KGC_ALLOC_FIXED));
+}
+
+void*
+jrealloc(void* mem, size_t sz)
+{
+	return (KGC_realloc(main_collector, mem, sz, KGC_ALLOC_FIXED));
 }
 
 void
-__free(void* mem)
+jfree(void* mem)
 {
-	(*Kaffe_GarbageCollectorInterface._free)(mem);
+	KGC_free(main_collector, mem);
 }
