@@ -12,15 +12,30 @@
 #ifndef __gc_h
 #define __gc_h
 
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
+#endif
+
+#if defined(HAVE_LIMITS_H)
+#include <limits.h>
+#endif
+
 #include "gtypes.h"
 
 /*
  * Default values for initial and maximum heap size and heap increment.
  */
+#define UNLIMITED_HEAP  (ULONG_MAX)
 #define	MIN_HEAPSIZE	(5*1024*1024)
-#define	MAX_HEAPSIZE	(64*1024*1024)
+#define	MAX_HEAPSIZE	(UNLIMITED_HEAP)
 #define	ALLOC_HEAPSIZE	(1024*1024)
 
+/*
+ * This macro sets the maximal value you can allocate in one chunk of memory. The type
+ * signed so we substract one bit.
+ */
+#define KGC_MAX_MALLOC_TYPE SIZE_MAX
+
 /* 
  * We do not support incremental collection as this time.
  */
@@ -46,6 +61,7 @@ typedef enum {
 	KGC_ALLOC_REFARRAY,
 	KGC_ALLOC_FINALIZEOBJECT,
 	KGC_ALLOC_JAVALOADER,
+	KGC_ALLOC_VMWEAKREF,
 
 	/* allocation types related to the translator engines */
 	KGC_ALLOC_JITCODE,
@@ -234,4 +250,7 @@ extern Collector* main_collector;
 #define gc_disableGC()	    KGC_disableGC(main_collector)
 
 extern const char* describeObject(const void* mem);
+
+Collector* 		initCollector(void);
+
 #endif
