@@ -2,9 +2,12 @@
  * gcFuncs.c
  * Methods to implement gc-related activities of objects and classes
  *
- * Copyright (c) 1996, 1997, 1998, 1999, 2004
+ * Copyright (c) 1996, 1997, 1998, 1999
  *      Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution
  * of this file.
  */
@@ -43,7 +46,6 @@
 #include "jni.h"
 #include "soft.h"
 #include "thread.h"
-#include "methodCache.h"
 #include "jvmpi_kaffe.h"
 #include "methodcalls.h"
 
@@ -59,7 +61,7 @@ static void
 destroyClass(Collector *collector, void* c)
 {
         int i;
-	int idx;
+	unsigned int idx;
 	Hjava_lang_Class* clazz = c;
 	constants* pool;
 
@@ -79,7 +81,7 @@ DBG(CLASSGC,
 	assert(clazz->state != CSTATE_COMPLETE || clazz->loader != 0);
 
 #if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_CLASS_UNLOAD) )
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_CLASS_UNLOAD) && clazz->state >= CSTATE_PREPARED)
 	{
 		JVMPI_Event ev;
 
@@ -118,9 +120,8 @@ DBG(CLASSGC,
 			if (!CLASS_IS_INTERFACE(clazz))
 			{
 				ncode = METHOD_NATIVECODE(m);
-#if defined(TRANSLATOR) && (defined (MD_UNREGISTER_JIT_EXCEPTION_INFO) || defined (JIT3))
 				if (METHOD_JITTED(m)) {
-#if defined(MD_UNREGISTER_JIT_EXCEPTION_INFO)
+#if defined(TRANSLATOR) && defined (MD_UNREGISTER_JIT_EXCEPTION_INFO)
 					MD_UNREGISTER_JIT_EXCEPTION_INFO (m->c.ncode.ncode_start,
 									  ncode,
 									  m->c.ncode.ncode_end);
@@ -137,7 +138,6 @@ DBG(CLASSGC,
 					}
 #endif
 				}
-#endif
 			}
 			utf8ConstRelease(m->name);
                         utf8ConstRelease(METHOD_SIG(m));
@@ -162,6 +162,8 @@ DBG(CLASSGC,
 		case CONSTANT_Utf8:
 			utf8ConstRelease(WORD2UTF(pool->data[idx]));
 			break;
+		default:
+			break;
 		}
 	}
 	/* free constant pool */
@@ -206,13 +208,18 @@ DBG(CLASSGC,
 		  if ((*impl_clazz)->interfaces[i] == clazz)
 		    {
 		      (*impl_clazz)->interfaces[i] = NULL;
-		      break;
+		      /* We cannot break here because there may exist
+		       * duplicates in the current list.
+		       */
 		    }
 	      }
 	    
 	    KFREE(clazz->implementors);
 	  }
 
+	if (clazz->interfaces)
+		gc_rm_ref(clazz->interfaces);
+
 	if( clazz->itable2dtable )
 	{
 	  for (i = 0; i < clazz->total_interface_len; i++) {
@@ -229,6 +236,7 @@ DBG(CLASSGC,
 	   * GC when the class becomes unused as it is only marked while
 	   * the class is being walked (see walkClass).
 	   */
+	  KGC_rmRef(collector, clazz->itable2dtable);
 	}
 	if (clazz->gc_layout != NULL && clazz->superclass != NULL &&
 	    clazz->superclass->gc_layout != clazz->gc_layout)
@@ -313,14 +321,6 @@ DBG(GCPRECISE,
 	  KGC_markObject(collector, gc_info, class->superclass);
         } 
 
-        /* We only need the interface array to be allocated.
-	 * We do not want to mark array's interfaces as the pointer is
-	 * static (see lookupArray). */
-	if (class->interfaces != NULL && CLASS_CNAME(class)[0] != '[')
-          {
-            KGC_markObject(collector, gc_info, class->interfaces);
-          }
-
 	if (class->itable2dtable != NULL)
 	  {
 	    unsigned int len = class->if2itable[class->total_interface_len];
@@ -353,6 +353,8 @@ DBG(GCPRECISE,
                 case CONSTANT_ResolvedString:
                         KGC_markObject(collector, gc_info, (void*)pool->data[idx]);
                         break;
+		default:
+			break;
                 }
         }
 
@@ -466,6 +468,7 @@ DBG(GCPRECISE,
         KGC_markObject(collector, gc_info, class->loader);
 	KGC_markObject(collector, gc_info, class->signers);
 	KGC_markObject(collector, gc_info, class->protectionDomain);
+	KGC_markObject(collector, gc_info, class->constructor);
 }
 
 /*****************************************************************************
@@ -492,6 +495,10 @@ walkRefArray(Collector* collector, void *gc_info, void* base, uint32 size UNUSED
    	if (lk != NULL && KGC_getObjectIndex(collector, lk) == KGC_ALLOC_LOCK)
 	  KGC_markObject(collector, gc_info, lk);
 
+DBG(GCPRECISE,
+        dprintf("walkRefArray `%s' (num=%d)\n", CLASS_CNAME(arr->vtable->class), ARRAY_SIZE(arr));
+   );
+
 	ptr = OBJARRAY_DATA(arr);
         /* mark class only if not a system class (which would be anchored
          * anyway.)  */
@@ -499,7 +506,7 @@ walkRefArray(Collector* collector, void *gc_info, void* base, uint32 size UNUSED
                 KGC_markObject(collector, gc_info, arr->vtable->class);
         }
 
-        for (i = ARRAY_SIZE(arr); --i>= 0; ) {
+        for (i = ARRAY_SIZE(arr); i > 0; i--) {
                 Hjava_lang_Object* el = *ptr++;
 		/*
 		 * NB: This would break if some objects (i.e. class objects)
@@ -681,17 +688,17 @@ initCollector(void)
 	DBG(INIT, dprintf("initCollector()\n"); );
 
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_JAVASTRING,
-	    stringWalk, KGC_OBJECT_NORMAL, stringDestroy, "j.l.String");
+	    stringWalk, stringDestroy, NULL, "j.l.String");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_NOWALK,
 	    NULL, KGC_OBJECT_NORMAL, NULL, "other-nowalk");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_NORMALOBJECT,
 	    walkObject, KGC_OBJECT_NORMAL, NULL, "obj-no-final");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_PRIMARRAY,
-	    NULL, KGC_OBJECT_NORMAL, NULL, "prim-arrays");
+	    NULL, finalizeObject, NULL, "prim-arrays");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_REFARRAY,
-	    walkRefArray, KGC_OBJECT_NORMAL, NULL, "ref-arrays");
+	    walkRefArray, finalizeObject, NULL, "ref-arrays");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_CLASSOBJECT,
-	    walkClass, KGC_OBJECT_NORMAL, destroyClass, "j.l.Class");
+	    walkClass, finalizeObject, destroyClass, "j.l.Class");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_FINALIZEOBJECT,
 	    walkObject, finalizeObject, NULL, "obj-final");
 	KGC_registerGcTypeByIndex(gc, KGC_ALLOC_JAVALOADER,
@@ -739,6 +746,7 @@ initCollector(void)
 	KGC_registerFixedTypeByIndex(gc, KGC_ALLOC_JIT_SLOTS, "jit-slots");
 	KGC_registerFixedTypeByIndex(gc, KGC_ALLOC_JIT_CODEBLOCK, "jit-codeblock");
 	KGC_registerFixedTypeByIndex(gc, KGC_ALLOC_JIT_LABELS, "jit-labels");
+	KGC_registerFixedTypeByIndex(gc, KGC_ALLOC_VMWEAKREF, "vm-weak-ref");
 
 	DBG(INIT, dprintf("initCollector() done\n"); );
 	return (gc);
