@@ -12,14 +12,15 @@
  * of this file.
  */
 
-#define	DBG(s)
+#define	IDBG(s) /* XXX */
 
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
+#include "debug.h"
 #include "gtypes.h"
-#include "inflate.h"
 #include "gc.h"
+#include "inflate.h"
 
 #define	WSIZE	0x8000
 
@@ -32,14 +33,14 @@ typedef struct _huft {
 	} v;
 } huft;
 
-static int inflate_codes(inflateInfo*, huft*, huft*, int, int);
+static int inflate_codes(inflateInfo*, huft*, huft*, size_t, size_t);
 static int inflate_stored(inflateInfo*);
 static int inflate_fixed(inflateInfo*);
 static int inflate_dynamic(inflateInfo*);
 static int inflate_block(inflateInfo*, int*);
 static int huft_free(huft*);
 static int inflate(inflateInfo*);
-static int huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint16* e, huft** t, int* m);
+static int huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint16* e, huft** t, size_t* m);
 
 /* Tables for deflate from PKZIP's appnote.txt. */
 static unsigned border[] = {    /* Order of the bit length code lengths */
@@ -108,15 +109,15 @@ static uint16 mask_bits[] = {
 
 static
 int
-inflate_codes(inflateInfo* pG, huft* tl, huft* td, int bl, int bd)
+inflate_codes(inflateInfo* pG, huft* tl, huft* td, size_t bl, size_t bd)
 {
-  register unsigned e;  /* table entry flag/number of extra bits */
-  unsigned n, d;        /* length and index for copy */
-  unsigned w;           /* current window position */
+  register size_t e;  /* table entry flag/number of extra bits */
+  size_t n, d;        /* length and index for copy */
+  size_t w;           /* current window position */
   huft *t;       /* pointer to table entry */
-  unsigned ml, md;      /* masks for bl and bd bits */
+  int ml, md;      /* masks for bl and bd bits */
   register uint32 b;       /* bit buffer */
-  register unsigned k;  /* number of bits in bit buffer */
+  register size_t k;  /* number of bits in bit buffer */
 
 
   /* make local copies of globals */
@@ -130,7 +131,7 @@ inflate_codes(inflateInfo* pG, huft* tl, huft* td, int bl, int bd)
   md = mask_bits[bd];
   while (1)                     /* do until end of block */
   {
-    NEEDBITS(pG, (unsigned)bl)
+    NEEDBITS(pG, bl)
     if ((e = (t = tl + ((unsigned)b & ml))->e) > 16)
       do {
         if (e == 99)
@@ -161,7 +162,7 @@ inflate_codes(inflateInfo* pG, huft* tl, huft* td, int bl, int bd)
       DUMPBITS(pG, e);
 
       /* decode distance of block to copy */
-      NEEDBITS(pG, (unsigned)bd)
+      NEEDBITS(pG, bd)
       if ((e = (t = td + ((unsigned)b & md))->e) > 16)
         do {
           if (e == 99)
@@ -212,14 +213,14 @@ static
 int
 inflate_stored(inflateInfo* pG)
 {
-  unsigned n;           /* number of bytes in block */
-  unsigned w;           /* current window position */
-  register uint32 b;       /* bit buffer */
-  register unsigned k;  /* number of bits in bit buffer */
+  size_t n;           /* number of bytes in block */
+  size_t w;           /* current window position */
+  register int32 b;       /* bit buffer */
+  register int k;  /* number of bits in bit buffer */
 
 
   /* make local copies of globals */
-  DBG(fprintf(stderr, "\nstored block"));
+  IDBG(dprintf("\nstored block"));
   b = pG->bb;                       /* initialize bit buffer */
   k = pG->bk;
   w = pG->wp;                       /* initialize window position */
@@ -267,7 +268,7 @@ int
 inflate_fixed(inflateInfo* pG)
 {
   /* if first time, set up tables for fixed blocks */
-  DBG(fprintf(stderr, "\nliteral block"));
+  IDBG(dprintf("\nliteral block"));
   if (pG->fixed_tl == 0)
   {
     int i;                /* temporary variable */
@@ -286,7 +287,7 @@ inflate_fixed(inflateInfo* pG)
     if ((i = huft_build(pG, l, 288, 257, cplens, cplext,
                         &pG->fixed_tl, &pG->fixed_bl)) != 0)
     {
-      pG->fixed_tl = 0;
+      pG->fixed_tl = NULL;
       return i;
     }
 
@@ -298,7 +299,7 @@ inflate_fixed(inflateInfo* pG)
                         &pG->fixed_td, &pG->fixed_bd)) > 1)
     {
       huft_free(pG->fixed_tl);
-      pG->fixed_tl = 0;
+      pG->fixed_tl = NULL;
       return i;
     }
   }
@@ -319,8 +320,8 @@ inflate_dynamic(inflateInfo* pG)
   unsigned n;           /* number of lengths to get */
   huft *tl;      /* literal/length code table */
   huft *td;      /* distance code table */
-  int bl;               /* lookup bits for tl */
-  int bd;               /* lookup bits for td */
+  size_t bl;            /* lookup bits for tl */
+  size_t bd;            /* lookup bits for td */
   unsigned nb;          /* number of bit length codes */
   unsigned nl;          /* number of literal/length codes */
   unsigned nd;          /* number of distance codes */
@@ -330,7 +331,7 @@ inflate_dynamic(inflateInfo* pG)
 
 
   /* make local bit buffer */
-  DBG(fprintf(stderr, "\ndynamic block"));
+  IDBG(dprintf("\ndynamic block"));
   b = pG->bb;
   k = pG->bk;
 
@@ -362,7 +363,7 @@ inflate_dynamic(inflateInfo* pG)
 
   /* build decoding table for trees--single level, 7 bit lookup */
   bl = 7;
-  i = huft_build(pG, ll, 19, 19, 0, 0, &tl, &bl);
+  i = huft_build(pG, ll, 19, 19, NULL, NULL, &tl, &bl);
   if (bl == 0)                        /* no bit lengths */
     i = 1;
   if (i)
@@ -523,18 +524,18 @@ inflate_new(void)
 {
 	inflateInfo* info;
 
-	info = KMALLOC(sizeof(inflateInfo));
+	info = gc_malloc(sizeof(inflateInfo), KGC_ALLOC_FIXED);
 	if (!info) {
-		return 0;
+		return NULL;
 	}
-	info->fixed_tl = 0;
-	info->fixed_td = 0;
+	info->fixed_tl = NULL;
+	info->fixed_td = NULL;
 	info->fixed_bl = 0;
 	info->fixed_bd = 0;
-	info->slide = KMALLOC(WSIZE);
+	info->slide = gc_malloc(WSIZE, KGC_ALLOC_FIXED);
 	if (!info->slide){
-		KFREE(info);
-		return 0;
+		gc_free(info);
+		return NULL;
 	}
 
 	return (info);
@@ -602,7 +603,7 @@ inflate(inflateInfo* pG)
   FLUSH(pG, pG->wp);
 
   /* return success */
-  DBG(fprintf(stderr, "\n%u bytes in Huffman tables (%d/entry)\n", h * sizeof(huft), sizeof(huft)));
+  IDBG(dprintf("\n%u bytes in Huffman tables (%d/entry)\n", h * sizeof(huft), sizeof(huft)));
   return 0;
 }
 
@@ -613,10 +614,10 @@ inflate_free(inflateInfo* pG)
     if (pG->fixed_tl != 0) {
       huft_free(pG->fixed_td);
       huft_free(pG->fixed_tl);
-      pG->fixed_td = pG->fixed_tl = 0;
+      pG->fixed_td = pG->fixed_tl = NULL;
     }
-    KFREE(pG->slide);
-    KFREE(pG);
+    gc_free(pG->slide);
+    gc_free(pG);
   }
 
   return 0;
@@ -629,17 +630,17 @@ inflate_free(inflateInfo* pG)
 
 static
 int
-huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint16* e, huft** t, int* m)
+huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint16* e, huft** t, size_t* m)
 {
   unsigned a;                   /* counter for codes of length k */
   unsigned c[BMAX+1];           /* bit length count table */
   unsigned el;                  /* length of EOB code (value 256) */
   unsigned f;                   /* i repeats in table every f entries */
-  int g;                        /* maximum code length */
+  size_t g;                     /* maximum code length */
   int h;                        /* table level */
   register unsigned i;          /* counter, current code */
   register unsigned j;          /* counter */
-  register int k;               /* number of bits in current code */
+  register size_t k;            /* number of bits in current code */
   int lx[BMAX+1];               /* memory for l[-1..BMAX-1] */
   int *l = lx+1;                /* stack of bits per table */
   register unsigned *p;         /* pointer into c[], b[], or v[] */
@@ -647,11 +648,11 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
   huft r;                /* table entry for structure assignment */
   huft *u[BMAX];         /* table stack */
   unsigned v[N_MAX];            /* values in order of bit length */
-  register int w;               /* bits before this table == (l * h) */
+  register size_t w;            /* bits before this table == (l * h) */
   unsigned x[BMAX+1];           /* bit offsets, then code stack */
   unsigned *xp;                 /* pointer into x */
   int y;                        /* number of dummy codes added */
-  unsigned z;                   /* number of entries in current table */
+  size_t z;                     /* number of entries in current table */
 
 
   /* Generate counts for each bit length */
@@ -663,7 +664,7 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
   } while (--i);
   if (c[0] == n)                /* null input--all zero length codes */
   {
-    *t = 0;
+    *t = NULL;
     *m = 0;
     return 0;
   }
@@ -716,8 +717,8 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
   p = v;                        /* grab values in bit order */
   h = -1;                       /* no tables yet--level -1 */
   w = l[-1] = 0;                /* no bits decoded yet */
-  u[0] = 0;   /* just to keep compilers happy */
-  q = 0;      /* ditto */
+  u[0] = NULL;   /* just to keep compilers happy */
+  q = NULL;      /* ditto */
   z = 0;                        /* ditto */
 
   /* go through the bit lengths (k already is bits in shortest code) */
@@ -733,7 +734,7 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
         w += l[h++];            /* add bits already decoded */
 
         /* compute minimum size table less than or equal to *m bits */
-        z = (z = g - w) > (unsigned)*m ? *m : z;        /* upper limit */
+        z = (z = g - w) > *m ? *m : z;        /* upper limit */
         if ((f = 1 << (j = k - w)) > a + 1)     /* try a k-w bit table */
         {                       /* too few codes for k-w bit table */
           f -= a + 1;           /* deduct codes from patterns left */
@@ -751,7 +752,7 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
         l[h] = j;               /* set table size in stack */
 
         /* allocate and link in new table */
-        if ((q = (huft *)KMALLOC((z + 1)*sizeof(huft))) ==
+        if ((q = (huft *)gc_malloc((z + 1)*sizeof(huft), KGC_ALLOC_FIXED)) ==
             0)
         {
           if (h)
@@ -760,7 +761,7 @@ huft_build(inflateInfo* pG, unsigned* b, unsigned n, unsigned s, uint16* d, uint
         }
         pG->hufts += z + 1;         /* track memory usage */
         *t = q + 1;             /* link to list for huft_free() */
-        *(t = &(q->v.t)) = 0;
+        *(t = &(q->v.t)) = NULL;
         u[h] = ++q;             /* table starts after link */
 
         /* connect to last table, if there is one */
@@ -828,7 +829,7 @@ huft_free(huft* t)
   while (p != 0)
   {
     q = (--p)->v.t;
-    KFREE(p);
+    gc_free(p);
     p = q;
   }
   return 0;
