@@ -74,14 +74,17 @@ enum {
 /* For JIT3 compatibility */
 #define	begin_func_sync()	prepare_function_call()
 #define	end_func_sync()		fixup_function_call()
+#define begin_sync()		end_sub_block()
+#define end_sync()
+
 
 /* -------------------------------------------------------------------- */
 /* Conditional monitors */
 
 #define	monitor_enter() \
-	(meth->accflags & ACC_SYNCHRONISED ? mon_enter(meth, local(0)) : 0)
+	(meth->accflags & ACC_SYNCHRONISED ? mon_enter(meth, local(0)) : (void)0)
 #define	monitor_exit()	\
-	(meth->accflags & ACC_SYNCHRONISED ? mon_exit(meth, local(0)) : 0)
+	(meth->accflags & ACC_SYNCHRONISED ? mon_exit(meth, local(0)) : (void)0)
 
 /* -------------------------------------------------------------------- */
 /* Instruction formats */
