@@ -41,6 +41,7 @@ void move_float(SlotInfo*, SlotInfo*);
 void move_double(SlotInfo*, SlotInfo*);
 
 void add_int(SlotInfo*, SlotInfo*, SlotInfo*);
+void and_int_const(SlotInfo*, SlotInfo*, jint);
 void sub_int(SlotInfo*, SlotInfo*, SlotInfo*);
 void mul_int(SlotInfo*, SlotInfo*, SlotInfo*);
 void div_int(SlotInfo*, SlotInfo*, SlotInfo*);
@@ -122,6 +123,7 @@ void call(SlotInfo*);
 void call_indirect_method(Method*);
 void call_soft(void*);
 void ret(void);
+void exit_method(void);
 
 label* build_code_ref(uint8*, uintp);
 label* reference_code_label(uintp);
@@ -245,4 +247,7 @@ void softcall_checkarraystore(SlotInfo*, SlotInfo*);
 void softcall_addreference(SlotInfo*, SlotInfo*);
 void softcall_addreference_static(void*, SlotInfo*);
 void softcall_abstractmethod(Utf8Const*, Utf8Const*);
+void softcall_exit_method(Method *meth);
+void softcall_enter_method(SlotInfo *obj, Method *meth);
+
 #endif
