@@ -202,6 +202,20 @@ prologue(Method* meth)
 
 	/* Emit prologue code */
 	slot_const_const(NULL, (jword)l, (jword)meth, HAVE_prologue, Tnull);
+
+#if defined(ENABLE_JVMPI)
+	{
+		SlotInfo *tmp;
+
+		slot_alloctmp(tmp);
+		if( METHOD_IS_STATIC(meth) )
+			move_ref_const(tmp, NULL);
+		else
+			move_ref(tmp, local(0));
+		softcall_enter_method(tmp, meth);
+		slot_freetmp(tmp);
+	}
+#endif
 }
 
 void
@@ -225,6 +239,14 @@ epilogue(void)
 	slot_slot_slot(NULL, NULL, NULL, HAVE_epilogue, Tnull);
 }
 
+void
+exit_method(void)
+{
+#if defined(ENABLE_JVMPI)
+	softcall_exit_method(globalMethod);
+#endif
+}
+
 void
 ret(void)
 {
@@ -1084,7 +1106,7 @@ neg_long(SlotInfo* dst, SlotInfo* src)
 	adc_int_const(HSLOT(dst), HSLOT(src), 0);
 	neg_int(HSLOT(dst), HSLOT(dst));
 #else
-	ABORT();
+	KAFFEVM_ABORT();
 #endif
 }
 
@@ -2343,7 +2365,7 @@ store_const_offset_byte(SlotInfo* dst, jint offset, jint val)
 /*									   */
 
 void
-pusharg_int_const(int val, int idx)
+pusharg_int_const(jint val, int idx)
 {
 #if defined(HAVE_pusharg_int_const)
 	if (HAVE_pusharg_int_const_rangecheck(val)) {
@@ -2713,7 +2735,7 @@ returnarg_double(SlotInfo* src)
 /*									   */
 
 label*
-reference_label(int32 i, int32 n)
+reference_label(int32 i UNUSED, int32 n)
 {
 	label* l;
 
@@ -2779,7 +2801,7 @@ table_code_label(SlotInfo* dst)
 
 #if defined(HAVE_set_label)
 void
-set_label(int i, int n)
+set_label(int i UNUSED, int n)
 {
 	assert(n < MAXLABTAB);
 	if (labtab[n] == 0) {
@@ -3130,7 +3152,6 @@ cvt_double_int(SlotInfo* dst, SlotInfo* src)
 #elif defined(HAVE_cvt_double_int_ieee)
 	{
 	  SlotInfo *tmp;
-	  int i;
 	  
 	  end_sub_block();
 	  slot_alloc2tmp(tmp);
@@ -3238,7 +3259,7 @@ cvt_int_short(SlotInfo* dst, SlotInfo* src)
 void
 softcall_breakpoint()
 {
-	ABORT();
+	KAFFEVM_ABORT();
 }
 
 /* ----------------------------------------------------------------------- */
@@ -3317,7 +3338,7 @@ build_call_frame(Utf8Const* sig, SlotInfo* obj, int sp_idx)
 
 		case 'V':
 		default:
-			ABORT();
+			KAFFEVM_ABORT();
 		}
 
 		sp_idx--;
@@ -3687,8 +3708,24 @@ softcall_exit_method(Method *meth)
 	popargs();
 	end_func_sync();
 }
-#endif
 
+void
+softcall_enter_method(SlotInfo *obj, Method *meth)
+{
+        begin_func_sync();
+#if defined(PUSHARG_FORWARDS)
+	pusharg_ref(obj, 0);
+	pusharg_ref_const(meth, 1);
+#else
+	pusharg_ref_const(meth, 1);
+	pusharg_ref(obj, 0);
+#endif
+	call_soft(soft_enter_method);
+	popargs();
+	end_func_sync();
+}
+
+#endif
 
 #if defined(GC_INCREMENTAL)
 void
