@@ -49,6 +49,7 @@
 #include "stats.h"
 #include "support.h"
 #include "native-wrapper.h"
+#include "jvmpi_kaffe.h"
 
 /*
  * Define information about this engine.
@@ -124,7 +125,7 @@ struct {
 } jitStats;
 
 static void generateInsnSequence(codeinfo* codeInfo);
-static void checkCaughtExceptions(Method* meth, int pc);
+static void checkCaughtExceptions(Method* meth, unsigned int pc);
 
 void	endBlock(sequence*);
 void	startBlock(sequence*);
@@ -133,6 +134,22 @@ void	startSubBlock(sequence*);
 void	cancelNoWriteback(void);
 jlong	currentTime(void);
 
+static JTHREAD_JMPBUF JIT_jumpExitWithOOM;
+
+void KaffeJIT_exitWithOOM()
+{
+  JTHREAD_LONGJMP(JIT_jumpExitWithOOM, 1);
+}
+
+bool KaffeJIT_setupExitWithOOM(struct _errorInfo* einfo)
+{
+  if (!JTHREAD_SETJMP(JIT_jumpExitWithOOM))
+    return false;
+
+  postOutOfMemory(einfo);
+  return true;
+}
+
 /*
  * By default, we comply with the Java spec and turn stack overflow checks
  * on.  Note that this involves a noticeable performance penalty.  If you
@@ -157,7 +174,7 @@ checkStackOverflow(void)
 	if (thread_data->needOnStack == STACK_LOW) {
 		dprintf(
 			"Panic: unhandled StackOverflowError()\n");
-		ABORT();
+		KAFFEVM_ABORT();
 	}
 	
 	{
@@ -190,7 +207,7 @@ translate(Method* meth, errorInfo *einfo)
 	SlotInfo* mtable;
 
 	bytecode* base;
-	int len;
+	unsigned int len;
 	callInfo cinfo;
 	fieldInfo finfo;
 	Hjava_lang_Class* crinfo;
@@ -242,6 +259,8 @@ DBG(MOREJIT,
 	/* Scan the code and determine the basic blocks */
 	success = analyzeMethod(meth, &codeInfo, einfo); 
 	if (success == false) {
+		if (METHOD_TRANSLATED(meth))
+			success = true;
 		goto done3;
 	}
 
@@ -254,6 +273,12 @@ DBG(MOREJIT,
 	enterTranslator();
 	startTiming(&jit_time, "jittime");
 
+	if (KaffeJIT_setupExitWithOOM(einfo))
+	  {
+	    success = false;
+	    goto done2;
+	  }
+
 #if defined(KAFFE_PROFILER)
 	if (profFlag) {
 		static int init = 0;
@@ -426,6 +451,17 @@ done2:
 	leaveTranslator();
 done3:
 	unlockClass(meth->class);
+	
+#if defined(ENABLE_JVMPI)
+	if (success && JVMPI_EVENT_ISENABLED(JVMPI_EVENT_COMPILED_METHOD_LOAD) )
+	  {
+	    JVMPI_Event ev;
+
+	    jvmpiFillMethodLoad(&ev, meth);
+	    jvmpiPostEvent(&ev);
+	  }
+#endif
+
 	return (success);
 }
 
@@ -440,9 +476,9 @@ bool
 finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code, errorInfo *einfo)
 {
 #if defined(CALLTARGET_ALIGNMENT)
-	int align = CALLTARGET_ALIGNMENT;
+	unsigned int align = CALLTARGET_ALIGNMENT;
 #else
-	int align = 0;
+	unsigned int align = 0;
 #endif
 #if defined(MD_JIT_EXCEPTION_INFO_LENGTH)
 	int exc_len = MD_JIT_EXCEPTION_INFO_LENGTH;
@@ -474,7 +510,7 @@ finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code, errorInfo *einfo)
 	jch->code_len = CODEPC;
 	/* align entry point if so desired */
 	if (align != 0 && (unsigned long)jch->code_start % align != 0) {
-		int pad = (align - (unsigned long)jch->code_start % align);
+		unsigned int pad = (align - (unsigned long)jch->code_start % align);
 		
 		assert(pad <= align - ALIGNMENT_OF_SIZE(sizeof(jdouble)));
 		
@@ -524,7 +560,7 @@ getInsnPC(int pc, codeinfo* codeInfo, nativeCodeInfo* code)
 void
 installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 {
-	int i;
+	unsigned int i;
 	jexceptionEntry* e;
 	jitCodeHeader *jch;
 	void *tramp;
@@ -630,6 +666,13 @@ initInsnSequence(Method* meth, int codesize, int localsz, int stacksz,
 	return true;
 }
 
+/*
+ */
+void KaffeJIT_cleanupInsnSequence()
+{
+  initSeq();
+}
+
 /*
  * Generate instructions from current set of sequences.
  */
@@ -690,7 +733,7 @@ startBlock(sequence* s)
  * Start a new basic sub-block.
  */
 void
-startSubBlock(sequence* s)
+startSubBlock(sequence* s UNUSED)
 {
 	int i;
 
@@ -709,7 +752,7 @@ startSubBlock(sequence* s)
  * Fixup after a function call.
  */
 void
-fixupFunctionCall(sequence* s)
+fixupFunctionCall(sequence* s UNUSED)
 {
 	int i;
 
@@ -880,9 +923,9 @@ cancelNoWriteback(void)
  */
 static
 void 
-checkCaughtExceptions(Method* meth, int pc)
+checkCaughtExceptions(Method* meth, unsigned int pc)
 {
-	int i;
+	unsigned int i;
 
 	willcatch.BADARRAYINDEX = false;
 	willcatch.NULLPOINTER = false;
@@ -935,7 +978,7 @@ static profiler_click_t click_divisor;
 static FILE *prof_output;
 
 static int
-profilerClassStat(Hjava_lang_Class *clazz, void *param)
+profilerClassStat(Hjava_lang_Class *clazz, void *param UNUSED)
 {
 	Method *meth;
 	int mindex;
