@@ -23,6 +23,7 @@
 #define	define_insn_alias(code)	case code :
 #define	define_wide_insn(code)	break;					\
 				case code :
+#define define_insn_fini
 
 /* Stack */
 #define	push(_i)		stackno -= (_i)
@@ -142,8 +143,8 @@
 #define	SOFT_ADDREFERENCE_STATIC(_f, _t)
 #endif
 
-struct _methods;
-bool translate(struct _methods*, errorInfo*);
+struct _jmethodID;
+bool translate(struct _jmethodID*, errorInfo*);
 
 typedef struct _jitCodeHeader {
 	unsigned long flags;
@@ -166,6 +167,10 @@ bool initInsnSequence(Method *meth, int codesize, int localsz, int stacksz,
 bool finishInsnSequence(struct codeinfo*, nativeCodeInfo*, errorInfo*);
 void installMethodCode(struct codeinfo*, Method*, nativeCodeInfo*);
 
+extern void KaffeJIT_cleanupInsnSequence();
+extern void KaffeJIT_exitWithOOM();
+extern bool KaffeJIT_setupExitWithOOM(errorInfo *einfo);
+
 typedef struct {
         bool BADARRAYINDEX;
         bool NULLPOINTER;
