@@ -31,6 +31,10 @@ engine_buildTrampoline (Method *meth, void **where, errorInfo *einfo);
 void
 engine_callMethod (callMethodInfo *call);
 
+void
+engine_dispatchException (uintp framePointer, uintp handler, 
+			  struct Hjava_lang_Throwable *throwable);
+
 /*
  * Fix up trampoline.
  */
