@@ -1,9 +1,12 @@
 /* machine.c
  * Translate the Kaffe instruction set to the native one.
  *
- * Copyright (c) 1996-1999, 2003, 2004
+ * Copyright (c) 1996-1999
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2003, 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *  
  * Cross-language profiling changes contributed by
  * the Flux Research Group, Department of Computer Science,
  * University of Utah, http://www.cs.utah.edu/flux/
@@ -48,7 +51,6 @@
 #include "soft.h"
 #include "thread.h"
 #include "itypes.h"
-#include "methodCache.h"
 #include "support.h"
 #include "xprofiler.h"
 #if defined(KAFFE_FEEDBACK)
@@ -60,6 +62,7 @@
 #include "jvmpi_kaffe.h"
 #include "kaffe_jni.h"
 #include "native-wrapper.h"
+#include "stats.h"
 
 const char* engine_name = "Just-in-time v3";
 
@@ -134,6 +137,22 @@ int profFlag;
 static void printProfilerStats(void);
 #endif
 
+JTHREAD_JMPBUF JIT3_jumpExitWithOOM;
+
+void KaffeJIT3_exitWithOOM()
+{
+  JTHREAD_LONGJMP(JIT3_jumpExitWithOOM, 1);
+}
+
+jboolean KaffeJIT3_setupExitWithOOM(struct _errorInfo* einfo)
+{
+  if (!JTHREAD_SETJMP(JIT3_jumpExitWithOOM))
+    return false;
+
+  postOutOfMemory(einfo);
+  return true;
+}
+
 /*
  * Translate a method into native code.
  *
@@ -202,6 +221,11 @@ translate(Method* xmeth, errorInfo* einfo)
 	/* Scan the code and determine the basic blocks */
 	success = analyzeMethod(xmeth, &mycodeInfo, einfo);
 	if (success == false) {
+		/* It may happen that we already have translated it
+		 * by implicit recursion in the verifier.
+		 */
+		if (METHOD_TRANSLATED(xmeth))
+		  success = true;
 		goto done3;
 	}
 
@@ -214,6 +238,8 @@ translate(Method* xmeth, errorInfo* einfo)
 	 */
 	enterTranslator();
 
+	startTiming(&fulljit, "JIT translation");
+
 	if (Kaffe_JavaVMArgs.enableVerboseJIT) {
 		tms = currentTime();
 	}
@@ -264,6 +290,13 @@ DBG(MOREJIT,
 		maxArgs += 1;
 	}
 
+
+	if (KaffeJIT3_setupExitWithOOM(einfo))
+	  {
+	    success = false;
+	    goto oom_error;
+	  }
+
 DBG(MOREJIT,
 	dprintf("Method: %s.%s%s\n", CLASS_CNAME(xmeth->class), xmeth->name->data, METHOD_SIGD(xmeth));
 	for (i = 0; i < maxLocal; i++) {
@@ -384,7 +417,11 @@ SCHK(			sanityCheck();				)
 	{
 		success = false;
 	}
+	goto done;
 
+oom_error:;
+	KaffeJIT3_cleanupInsnSequence();
+	
 done:;
 	KaffeJIT3_resetLabels();
 	KaffeJIT3_resetConstants();
@@ -420,6 +457,8 @@ DBG(MOREJIT,
 		       METHOD_NATIVECODE(xmeth), xmeth);
 	}
 
+	stopTiming(&fulljit);
+
 	leaveTranslator();
 #if defined(KAFFE_FEEDBACK)
 	if( kaffe_feedback_file )
@@ -429,47 +468,34 @@ done3:;
 	unlockClass(xmeth->class);
 
 #if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_COMPILED_METHOD_LOAD) &&
-	    success )
-	{
-		JVMPI_Event ev;
+	if (success && JVMPI_EVENT_ISENABLED(JVMPI_EVENT_COMPILED_METHOD_LOAD) )
+	  {
+	    JVMPI_Event ev;
 
-		ev.event_type = JVMPI_EVENT_COMPILED_METHOD_LOAD;
-		ev.u.compiled_method_load.method_id = xmeth;
-		ev.u.compiled_method_load.code_addr =
-			METHOD_NATIVECODE(xmeth);
-		ev.u.compiled_method_load.code_size =
-			(uintp)xmeth->c.ncode.ncode_end - (uintp)xmeth->c.ncode.ncode_start;
-		if( xmeth->lines )
-		{
-			JVMPI_Lineno *jvmpi_lineno = NULL;
-			int lpc;
-			
-			jvmpi_lineno = alloca(sizeof(JVMPI_Lineno) *
-					      xmeth->lines->length);
-			for( lpc = 0; lpc < xmeth->lines->length; lpc++ )
-			{
-				jvmpiConvertLineno(&jvmpi_lineno[lpc],
-						   &xmeth->lines->entry[lpc],
-						   METHOD_NATIVECODE(xmeth));
-			}
-			ev.u.compiled_method_load.lineno_table_size =
-				xmeth->lines->length;
-			ev.u.compiled_method_load.lineno_table =
-				jvmpi_lineno;
-		}
-		else
-		{
-			ev.u.compiled_method_load.lineno_table_size = 0;
-			ev.u.compiled_method_load.lineno_table = NULL;
-		}
-		jvmpiPostEvent(&ev);
-	}
+	    jvmpiFillMethodLoad(&ev, xmeth);
+	    jvmpiPostEvent(&ev);
+	  }
 #endif
 
 	return (success);
 }
 
+void
+KaffeJIT3_cleanupInsnSequence()
+{
+  sequence *s;
+
+  for (s = firstSeq; s != currSeq; s = s->next)
+    {
+      if (s->func != doSpill)
+	continue;
+      
+      /* According to doSpill */
+      gc_free(s->u[1].smask);
+    } 
+  initSeq();
+}
+
 /*
  * Generate the code.
  */
@@ -505,7 +531,7 @@ finishInsnSequence(void* dummy UNUSED, nativeCodeInfo* code, errorInfo* einfo)
 	jch = (jitCodeHeader *)methblock;
 	jch->pool = (void *)(jch + 1);
 	jch->pool = (void *) ( ((unsigned long)(jch->pool) + const_align) & ~const_align);
-	jch->code_start = ((char *)jch->pool) + constlen;
+	jch->code_start = ((nativecode *)jch->pool) + constlen;
 	jch->code_len = CODEPC;
 	memcpy(jch->code_start, codeblock, jch->code_len);
 	gc_free(codeblock);
@@ -599,6 +625,7 @@ installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 				strlen(METHOD_SIGD(meth)) +
 				1,
 				KGC_ALLOC_JITTEMP);
+		assert(sym != NULL);
 		sprintf(sym,
 			"%s/%s%s",
 			CLASS_CNAME(meth->class),
@@ -652,11 +679,14 @@ installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 		struct debug_file *df = machine_debug_file;
 
 		/* Mark the start of this source file */
+		char *sourcefile = meth->class->sourcefile;
+		if (! sourcefile)
+			sourcefile = "Unknown source file";
 		addDebugInfo(df,
 			     DIA_IncludeFile,
 			     meth->class->name->data,
 			     meth->class->packageLength,
-			     meth->class->sourcefile,
+			     sourcefile,
 			     code->code,
 
 			     DIA_Function,
@@ -745,12 +775,15 @@ installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 		/*
 		 * No line debugging, but we'd like a symbol to show up anyways
 		 */
+                char *sourcefile = meth->class->sourcefile;
+                if (! sourcefile)
+                        sourcefile = "Unknown source file";
 		addDebugInfo(
 			machine_debug_file,
 			DIA_IncludeFile,
 			meth->class->name->data,
 			meth->class->packageLength,
-			meth->class->sourcefile,
+			sourcefile,
 			code->code,
 			
 			DIA_Function, meth, mm, 0, code->code, code->codelen,
@@ -959,7 +992,7 @@ SCHK(	sanityCheck();						)
 				break;
 
 			default:
-				ABORT();
+				KAFFEVM_ABORT();
 			}
 		}
 	}
@@ -1008,7 +1041,7 @@ SCHK(	sanityCheck();						)
 				break;
 
 			default:
-				ABORT();
+				KAFFEVM_ABORT();
 			}
 		}
 	}
@@ -1039,7 +1072,7 @@ SCHK(	sanityCheck();						)
 				break;
 
 			default:
-				ABORT();
+				KAFFEVM_ABORT();
 			}
 		}
 	}
@@ -1084,7 +1117,9 @@ createSpillMask(void)
 
 	c++; /* Add null slot on the end */
 	mem = gc_malloc(c * sizeof(SlotData*), KGC_ALLOC_JIT_SLOTS);
-
+	if (mem == NULL)
+	  KaffeJIT3_exitWithOOM();
+	
 	i = maxLocal + maxStack + tmpslot;
 	c = 0;
 	for (i--; i >= 0; i--) {
@@ -1154,123 +1189,6 @@ SCHK(	sanityCheck();						)
 SCHK(	sanityCheck();						)
 }
 
-static
-int
-sortSlots(const void* s1, const void* s2)
-{
-	localUse* lcluse1;
-	localUse* lcluse2;
-
-	lcluse1 = &codeInfo->localuse[*((SlotInfo* const*)s1) - localinfo];
-	lcluse2 = &codeInfo->localuse[*((SlotInfo* const*)s2) - localinfo];
-
-	if (lcluse1->use == lcluse2->use) {
-		return (0);
-	}
-	else if (lcluse1->use > lcluse2->use) {
-		return (-1);
-	}
-	else {
-		return (1);
-	}
-}
-
-static
-void
-bindSlotToGlobal(int lcl, int r, int type)
-{
-	kregs* reg;
-	SlotData* dslot;
-	int gtype;
-
-	reg = &reginfo[r];
-	dslot = localinfo[lcl].slot;
-
-	reg->ctype = type;
-	reg->type |= Rglobal;
-	reg->refs++;
-	reg->slot = dslot;
-	dslot->regno = r;
-
-	gtype = GL_ISGLOBAL;
-
-	/* If this is an argument then pre-load */
-	if (lcl < maxArgs) {
-		gtype |= GL_PRELOAD;
-	}
-	/* If this slot is never written note that the global is
-	 * read only.
-	 */
-	if (codeInfo->localuse[lcl].write == -1) {
-		gtype |= GL_RONLY;
-	}
-	setGlobal(dslot, gtype);
-}
-
-/*
- * Setup global registers
- */
-void
-setupGlobalRegisters(void)
-{
-#if defined(NR_GLOBALS)
-	SlotInfo** slots;
-	int j;
-	int max;
-
-	/* If we don't have any code info we can't do any global
-	 * optimization
-	 */
-	if ((codeInfo == 0) || (codeInfo->localuse == 0)) {
-		return;
-	}
-
-	/* Allocate an array for the slot pointers and copy them in */
-	slots = gc_malloc((1+maxLocal) * sizeof(SlotInfo*), KGC_ALLOC_JIT_SLOTS);
-	for (j = 0; j < maxLocal; j++) {
-		slots[j] = &localinfo[j];
-	}
-
-	/* Sort the slots to the most used is first */
-	qsort(slots, maxLocal, sizeof(SlotInfo*), sortSlots);
-
-	/* Allocate the slots to globals */
-	max = NR_GLOBALS;
-	for (j = 0; j < maxLocal && max > 0; j++) {
-		int k;
-		int i;
-		localUse* lcl;
-
-		i = slots[j] - localinfo;
-		lcl = &codeInfo->localuse[i];
-
-		for (k = 0; k < MAXREG; k++) {
-			kregs* reg;
-
-			reg = &reginfo[k];
-			if ((reg->flags & Rglobal) == 0) {
-				/* Not a global */
-			}
-			else if ((reg->type & Rglobal) != 0) {
-				/* Already used */
-			}
-			else if (lcl->type == TINT && (reg->type & Rint) != 0) {
-				bindSlotToGlobal(i, k, Rint);
-				max--;
-				break;
-			}
-			else if (lcl->type == TOBJ && (reg->type & Rref) != 0) {
-				bindSlotToGlobal(i, k, Rref);
-				max--;
-				break;
-			}
-		}
-	}
-
-	gc_free(slots);
-#endif
-}
-
 void
 setupArgumentRegisters(void)
 {
@@ -1427,6 +1345,8 @@ newFakeCall(void* func, uintp currpc)
 		fc = KGC_malloc(main_collector,
 				sizeof(fakeCall),
 				KGC_ALLOC_JIT_FAKE_CALL);
+		if (fc == NULL)
+		  KaffeJIT3_exitWithOOM();
 	}
 #if defined(HAVE_branch_and_link)
 	fc->parent = findFakeCall(func);
