@@ -2,7 +2,7 @@
  * jni-callmethod.c
  * Java Native Interface - CallXMethod JNI functions.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
  * Copyright (c) 2004
@@ -36,8 +36,8 @@ getMethodFunc (Method* meth, Hjava_lang_Object *obj)
     implementors = meth->class->implementors;
     clazz = OBJECT_CLASS(obj);
 
-    assert (implementors != NULL && clazz->impl_index <= (uintp)implementors[0]);
-		
+    assert (implementors != NULL && clazz->impl_index <= (int)implementors[0]);
+
     return implementors[clazz->impl_index][meth->idx + 1]; 	
   } else {
     return meth->idx >= 0 ? obj->vtable->method[meth->idx] : METHOD_NATIVECODE (meth);
@@ -57,16 +57,24 @@ getMethodFunc (Method* meth, Hjava_lang_Object *obj)
 void
 KaffeJNI_CallVoidMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING_VOID();
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, NULL);
+  /*
+   * callMethodA will have to unveil the objects contained in the argument lists.
+   * This is not really pretty but more efficient because we do not parse the signature
+   * twice.
+   */
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, NULL);
 
   END_EXCEPTION_HANDLING();
 }
@@ -88,16 +96,24 @@ KaffeJNI_CallVoidMethod(JNIEnv* env, jobject obj, jmethodID meth, ...)
 void
 KaffeJNI_CallVoidMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING_VOID();
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, NULL, 0);
+  /*
+   * callMethodA will have to unveil the objects contained in the argument lists.
+   * This is not really pretty but more efficient because we do not parse the signature
+   * twice.
+   */
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, NULL, 0);
 
   END_EXCEPTION_HANDLING();
 }
@@ -112,6 +128,7 @@ KaffeJNI_CallNonvirtualObjectMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
 
@@ -119,7 +136,8 @@ KaffeJNI_CallNonvirtualObjectMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  obj_local = unveil(obj);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   ADD_REF(retval.l);
   END_EXCEPTION_HANDLING();
@@ -147,14 +165,16 @@ KaffeJNI_CallNonvirtualObjectMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
-
+  
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   ADD_REF(retval.l);
   END_EXCEPTION_HANDLING();
@@ -171,14 +191,16 @@ KaffeJNI_CallNonvirtualBooleanMethodV(JNIEnv* env UNUSED, jobject obj, jclass cl
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jboolean) retval.i);
@@ -205,14 +227,16 @@ KaffeJNI_CallNonvirtualBooleanMethodA(JNIEnv* env UNUSED, jobject obj, jclass cl
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jboolean) retval.i);
@@ -227,14 +251,16 @@ KaffeJNI_CallNonvirtualByteMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jbyte) retval.i);
@@ -261,14 +287,16 @@ KaffeJNI_CallNonvirtualByteMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jbyte) retval.i);
@@ -284,14 +312,16 @@ KaffeJNI_CallNonvirtualCharMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jchar) retval.i);
@@ -318,14 +348,16 @@ KaffeJNI_CallNonvirtualCharMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jchar) retval.i);
@@ -341,14 +373,16 @@ KaffeJNI_CallNonvirtualShortMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jshort) retval.i);
@@ -375,14 +409,16 @@ KaffeJNI_CallNonvirtualShortMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jshort) retval.i);
@@ -398,14 +434,16 @@ KaffeJNI_CallNonvirtualIntMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls UN
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.i);
@@ -432,14 +470,16 @@ KaffeJNI_CallNonvirtualIntMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls UN
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
+  obj_local = unveil(obj);
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.i);
@@ -451,19 +491,24 @@ KaffeJNI_CallNonvirtualIntMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls UN
  */
 
 jobject
+
 KaffeJNI_CallObjectMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
+
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   ADD_REF(retval.l);
   END_EXCEPTION_HANDLING();
@@ -491,16 +536,19 @@ jobject
 KaffeJNI_CallObjectMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.l);
@@ -515,16 +563,19 @@ jboolean
 KaffeJNI_CallBooleanMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jboolean) retval.i);
@@ -550,16 +601,19 @@ jboolean
 KaffeJNI_CallBooleanMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jboolean) retval.i);
@@ -574,16 +628,19 @@ jbyte
 KaffeJNI_CallByteMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+   o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jbyte) retval.i);
@@ -609,10 +666,13 @@ jbyte
 KaffeJNI_CallByteMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
@@ -633,16 +693,19 @@ jchar
 KaffeJNI_CallCharMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jchar) retval.i);
@@ -668,16 +731,19 @@ jchar
 KaffeJNI_CallCharMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jchar) retval.i);
@@ -692,16 +758,19 @@ jshort
 KaffeJNI_CallShortMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return ((jshort) retval.i);
@@ -727,16 +796,19 @@ jshort
 KaffeJNI_CallShortMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return ((jshort) retval.i);
@@ -751,16 +823,19 @@ jint
 KaffeJNI_CallIntMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.i);
@@ -786,16 +861,19 @@ jint
 KaffeJNI_CallIntMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.i);
@@ -810,16 +888,19 @@ jlong
 KaffeJNI_CallLongMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.j);
@@ -845,16 +926,19 @@ jlong
 KaffeJNI_CallLongMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.j);
@@ -869,16 +953,19 @@ jfloat
 KaffeJNI_CallFloatMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.f);
@@ -904,16 +991,19 @@ jfloat
 KaffeJNI_CallFloatMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.f);
@@ -928,16 +1018,19 @@ jdouble
 KaffeJNI_CallDoubleMethodV(JNIEnv* env UNUSED, jobject obj, jmethodID meth, va_list args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
+  KaffeVM_callMethodV(m, getMethodFunc (m, o), o, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.d);
@@ -963,16 +1056,19 @@ jdouble
 KaffeJNI_CallDoubleMethodA(JNIEnv* env UNUSED, jobject obj, jmethodID meth, jvalue* args)
 {
   jvalue retval;
-  Hjava_lang_Object* o = (Hjava_lang_Object*)obj;
+  jobject obj_local;
+  Hjava_lang_Object* o;
   Method* m = (Method*)meth;
 
   BEGIN_EXCEPTION_HANDLING(0);
+  obj_local = unveil(obj);
+  o = (Hjava_lang_Object*)obj_local;
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
 
-  KaffeVM_callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, getMethodFunc (m, o), o, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.d);
@@ -988,14 +1084,16 @@ KaffeJNI_CallNonvirtualLongMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.j);
@@ -1022,14 +1120,16 @@ KaffeJNI_CallNonvirtualLongMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls U
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.j);
@@ -1045,14 +1145,16 @@ KaffeJNI_CallNonvirtualFloatMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.f);
@@ -1079,14 +1181,16 @@ KaffeJNI_CallNonvirtualFloatMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.f);
@@ -1102,14 +1206,16 @@ KaffeJNI_CallNonvirtualDoubleMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, &retval);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, &retval);
 
   END_EXCEPTION_HANDLING();
   return (retval.d);
@@ -1136,14 +1242,16 @@ KaffeJNI_CallNonvirtualDoubleMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls
 {
   jvalue retval;
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING(0);
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, &retval, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, &retval, 0);
 
   END_EXCEPTION_HANDLING();
   return (retval.d);
@@ -1158,14 +1266,16 @@ void
 KaffeJNI_CallNonvirtualVoidMethodV(JNIEnv* env UNUSED, jobject obj, jclass cls UNUSED, jmethodID meth, va_list args)
 {
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING_VOID();
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj, args, NULL);
+  KaffeVM_callMethodV(m, METHOD_NATIVECODE(m), obj_local, args, NULL);
 
   END_EXCEPTION_HANDLING();
 }
@@ -1188,14 +1298,16 @@ void
 KaffeJNI_CallNonvirtualVoidMethodA(JNIEnv* env UNUSED, jobject obj, jclass cls UNUSED, jmethodID meth, jvalue* args)
 {
   Method* m = (Method*)meth;
+  jobject obj_local;
 
   BEGIN_EXCEPTION_HANDLING_VOID();
 
   if (METHOD_IS_STATIC(m)) {
     throwException(NoSuchMethodError(m->name->data));
   }
+  obj_local = unveil(obj);
 
-  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj, args, NULL, 0);
+  KaffeVM_callMethodA(m, METHOD_NATIVECODE(m), obj_local, args, NULL, 0);
 
   END_EXCEPTION_HANDLING();
 }
