@@ -65,10 +65,24 @@ stringJava2CBuf(const Hjava_lang_String* js, char* cs, int len)
 	if (len > STRING_SIZE(js)) {
 		len = STRING_SIZE(js);
 	}
-	cs[len] = 0;
+
 	while (--len >= 0) {
-		*cs++ = (char)*chrs++;
+	  if (*chrs >= 0x0001 && *chrs <= 0x007F) {
+	    *cs++ = *chrs++ & 0x7F;
+	  }
+	  else if (*chrs >= 0x0080 && *chrs <= 0x07FF) {
+	    *cs++ = 0xC0 | ((*chrs >> 6) & 0x1F);
+	    *cs++ = 0x80 | (*chrs++ & 0x3F);
+	  }
+	  else {
+	    *cs++ = 0xE0 | ((*chrs >> 12) & 0x0F);
+	    *cs++ = 0x80 | ((*chrs >> 6) & 0x3F);
+	    *cs++ = 0x80 | (*chrs++ & 0x3F);
+	  }
 	}
+	
+	*cs = 0;
+
 	return (cs);
 }
 
@@ -99,7 +113,7 @@ stringC2Java(const char* cs)
 	/* Return a string containing those chars */
 	string = stringCharArray2Java(ary, len);
 	if (ary != buf) {
-		KFREE(ary);
+		gc_free(ary);
 	}
 	return(string);
 }
@@ -185,7 +199,7 @@ utf8Const2JavaReplace(const Utf8Const *utf8, jchar from_ch, jchar to_ch)
 	/* Convert to a String object */
 	string = stringCharArray2Java(jc, uniLen);
 	if (jc != buf) {
-		KFREE(jc);
+		gc_free(jc);
 	}
 	return(string);
 }
@@ -222,7 +236,7 @@ stringJava2Utf8ConstReplace(Hjava_lang_String *str, jchar from, jchar to)
 	utf8buf = utf8ConstEncode(chars, slength);
 
 	if (chars != STRING_DATA(str)) {
-		KFREE(chars);
+		gc_free(chars);
 	}
 
 	if (utf8buf == 0) {
@@ -268,7 +282,7 @@ utf8ConstEqualJavaString(const Utf8Const *utf8, const Hjava_lang_String *string)
 
 /*
  * Define functions used by the string hashtable to resize itself.
- * The problem is that we may block in KCALLOC/KFREE and the gc may kick
+ * The problem is that we may block in gc_malloc/gc_free and the gc may kick
  * in.  The collector, however, must be able to call stringUninternString
  * while destroying strings.  If we held the lock while this is happening,
  * we would deadlock.
@@ -290,7 +304,7 @@ stringFree(const void *ptr)
 {
 	/* XXX assumes stringLock isn't acquired recursively (which it isn't) */
         unlockStaticMutex(&stringLock);	
-	KFREE((void *) ptr);
+	gc_free((void *) ptr);
 	lockStaticMutex(&stringLock);
 }
 
@@ -298,17 +312,17 @@ stringFree(const void *ptr)
  * Return the interned version of a String object.
  * May or may not be the same String.
  */
-const Hjava_lang_String *
+Hjava_lang_String *
 stringInternString(Hjava_lang_String *string)
 {
-	const Hjava_lang_String *temp;
+	Hjava_lang_String *temp;
 
 	/* Lock intern table */
 	lockStaticMutex(&stringLock);
 
 	/* See if string is already in the table */
 	if (hashTable != NULL) {
-		const Hjava_lang_String *string2;
+		Hjava_lang_String *string2;
 
 		if ((string2 = hashFind(hashTable, string)) != NULL) {
 			unlockStaticMutex(&stringLock);
@@ -413,12 +427,12 @@ stringCompare(const void *v1, const void *v2)
 Hjava_lang_String*
 stringCharArray2Java(const jchar *data, int len)
 {
-	Hjava_lang_String *string;
+        Hjava_lang_String *string;
 	HArrayOfChar *ary;
 	errorInfo info;
 
 	/* Lock intern table 
-	 * NB: we must not hold stringLock when we call KMALLOC/KFREE!
+	 * NB: we must not hold stringLock when we call gc_malloc/gc_free!
 	 */
 
 	/* Look for it already in the intern hash table */
@@ -451,7 +465,7 @@ stringCharArray2Java(const jchar *data, int len)
 		unlockStaticMutex(&stringLock);
 
 		if (fakeAry != (HArrayOfChar*)buf) {
-			KFREE(fakeAry);
+			gc_free(fakeAry);
 		}
 		if (string != NULL) {
 			return(string);
