@@ -15,6 +15,15 @@
  */
 
 #include "config.h"
+
+#if defined(HAVE_SYS_TYPES_H)
+#include <sys/types.h>
+#endif /* defined(HAVE_SYS_TYPES_H) */
+
+#if defined(HAVE_SYS_WAIT_H)
+#include <sys/wait.h>
+#endif /* defined(HAVE_SYS_WAIT_H) */
+
 #include "jthread.h"
 #include "jsignal.h"
 #include "xprofiler.h"
@@ -22,7 +31,7 @@
 /* For NOTIMEOUT */
 #include "jsyscall.h"
 /* For jlong */
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 /* For Hjava_lang_VMThread */
 #include "thread.h"
 #include "gc.h"
@@ -90,7 +99,7 @@ static fd_set readsPending;	/* fds we want to read from */
 static fd_set writesPending;	/* fds we want to write to */
 static KaffeNodeQueue* readQ[FD_SETSIZE];	/* threads blocked on read */
 static KaffeNodeQueue* writeQ[FD_SETSIZE];	/* threads blocked on write */
-static jbool blockingFD[FD_SETSIZE];            /* file descriptor which should 
+static jboolean blockingFD[FD_SETSIZE];            /* file descriptor which should 
 						   really block */
 static jmutex threadLock;	/* static lock to protect liveThreads etc. */
 static jmutex GClock;
@@ -2263,7 +2272,7 @@ retry:
 		} else {
 			maxWait = firstAlarm - curTime;
 		}
-		DBG(JTHREADDETAIL, dprintf("handleIO(sleep=%d) maxWait=%d\n", canSleep, maxWait); );
+		DBG(JTHREADDETAIL, dprintf("handleIO(sleep=%d) maxWait=%ld\n", canSleep, (long) maxWait); );
 	}
 
 #if USE_POLL
@@ -2475,11 +2484,11 @@ jcondvar_initialise(jcondvar *cv)
 	*cv = NULL;
 }
 
-jbool
+jboolean
 jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout)
 {
 	jthread *current = jthread_current();
-	jbool r;
+	jboolean r;
 
 	intsDisable();
 
@@ -2694,14 +2703,14 @@ restore_fds(void)
     	}
 }
 
-static void NONRETURNING
+static void
 restore_fds_and_exit()
 {
 	restore_fds();
 	/* technically, we should restore the original handler and rethrow
 	 * the signal.
 	 */
-	EXIT(-1);		/* XXX */
+	KAFFEVM_EXIT(-1);		/* XXX */
 }
 
 /*
@@ -2906,7 +2915,7 @@ jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout)
  * Threaded socket accept.
  */
 int
-jthreadedAccept(int fd, struct sockaddr* addr, int* len, 
+jthreadedAccept(int fd, struct sockaddr* addr, socklen_t* len, 
 		int timeout, int* out)
 {
 	/* absolute time at which time out is reached */
@@ -3069,7 +3078,7 @@ jthreadedWrite(int fd, const void* buf, size_t len, ssize_t *out)
  */
 int 
 jthreadedRecvfrom(int fd, void* buf, size_t len, int flags, 
-	struct sockaddr* from, int* fromlen, int timeout, ssize_t *out)
+	struct sockaddr* from, socklen_t* fromlen, int timeout, ssize_t *out)
 {
 	int r;
 	jlong deadline = 0;
