@@ -36,6 +36,19 @@ jthreadedClose(int fd)
 	return (rc);
 }
 
+static int
+jthreadedSocketShutdown(int fd)
+{
+        int rc = 0;
+
+	jthread_spinon(0);
+	if (shutdown(fd, 2) == -1)
+	  rc = errno;
+	jthread_spinoff(0);
+
+	return rc;
+}
+
 static int
 jthreadedListen(int fd, int l)
 {
@@ -228,7 +241,7 @@ jthreadedSetSockOpt(int a, int b, int c, const void* d, int e)
 }
 
 static int
-jthreadedGetSockOpt(int a, int b, int c, void* d, int* e)
+jthreadedGetSockOpt(int a, int b, int c, void* d, socklen_t* e)
 {
 	int rc = 0;
 
@@ -241,7 +254,7 @@ jthreadedGetSockOpt(int a, int b, int c, void* d, int* e)
 }
 
 static int
-jthreadedGetSockName(int a, struct sockaddr* b, int* c)
+jthreadedGetSockName(int a, struct sockaddr* b, socklen_t* c)
 {
 	int rc = 0;
 
@@ -254,7 +267,7 @@ jthreadedGetSockName(int a, struct sockaddr* b, int* c)
 }
 
 static int
-jthreadedGetPeerName(int a, struct sockaddr* b, int* c)
+jthreadedGetPeerName(int a, struct sockaddr* b, socklen_t* c)
 {
 	int rc = 0;
 
@@ -424,6 +437,7 @@ SystemCallInterface Kaffe_SystemCallInterface = {
         jthreadedGetSockName, 
         jthreadedGetPeerName, 
         jthreadedClose,
+	jthreadedSocketShutdown,
         jthreadedGetHostByName,
         jthreadedGetHostByAddr,
         jthreadedSelect,	
