@@ -59,64 +59,65 @@
 		r = 0;					\
 	}
 
-static 
-int
-waitForTimeout(int fd, int timeout){
-	fd_set rset;
-	struct timeval tv;
-	int ret;
+static int selectHelper(int n, fd_set *readfds, fd_set *writefds, fd_set *errfds, int timeout)
+{
+  struct timeval tv;
+  int ret;
 
-	FD_ZERO(&rset);
-	FD_SET(fd,&rset);
-	tv.tv_sec = timeout / 1000;
-	tv.tv_usec = (timeout % 1000) * 1000;
+  jthread_current()->interrupting = 0;
+  if (timeout == NOTIMEOUT)
+    {
+      ret = select(n, readfds, writefds, errfds, NULL);
+    }
+  else
+    {      
+      tv.tv_sec = timeout / 1000;
+      tv.tv_usec = (timeout % 1000) * 1000;
+      ret = select(n, readfds, writefds, errfds, &tv);
+    }
 
-	jthread_current()->interrupting = 0;
-	if (timeout == NOTIMEOUT) 
-		ret = select(fd+1,&rset,NULL,NULL,NULL);
-	else	
-		ret = select(fd+1,&rset,NULL,NULL,&tv);
-
-	if (ret == 0) 
-		errno = ETIMEDOUT;
-	else if (ret == -1)
-	{
-		errno = EINTR;
-		jthread_current()->interrupting = 1;
-	}
-
-	return (ret);
+  if (ret == 0)
+    errno = ETIMEDOUT;
+  else if (ret == -1)
+    {
+      errno = EINTR;
+      jthread_current()->interrupting = 1;
+    }
+  
+  return ret;
 }
 
-/* These two functions would need to be merged some time later.
- */
-static
-int waitForWritable(int fd, int timeout)
+static int
+waitForTimeout(int fd, int timeout)
 {
-	fd_set wset;
-	struct timeval tv;
-	int ret;
+  fd_set rset;
+  
+  FD_ZERO(&rset);
+  FD_SET(fd,&rset);
 
-	FD_ZERO(&wset);
-        FD_SET(fd,&wset);
-        tv.tv_sec = timeout / 1000;
-	tv.tv_usec = (timeout % 1000) * 1000;
+  return selectHelper(fd+1, &rset, NULL, NULL, timeout);
+}
 
-	jthread_current()->interrupting = 0;
-        if (timeout == NOTIMEOUT)
-		ret = select(fd+1,NULL,&wset,NULL,NULL);
-	else
-		ret = select(fd+1,NULL,&wset,NULL,&tv);
+static int
+waitForRW(int fd, int timeout)
+{
+  fd_set rwset;
+  
+  FD_ZERO(&rwset);
+  FD_SET(fd,&rwset);
+  
+  return selectHelper(fd+1, &rwset, &rwset, NULL, timeout);
+}
 
-	if (ret == 0)
-		errno = ETIMEDOUT;
-	else if (ret == -1)
-	{
-		errno = EINTR;
-		jthread_current()->interrupting = 1;
-	}
-
-	return (ret);
+static int
+waitForWritable(int fd, int timeout)
+{
+  fd_set wset;
+  
+  FD_ZERO(&wset);
+  FD_SET(fd,&wset);
+  
+  return selectHelper(fd+1, NULL, &wset, NULL, timeout);
 }
 
 static
@@ -173,6 +174,17 @@ jthreadedClose(int fd)
 	return (rc);
 }
 
+static int
+jthreadedSocketShutdown(int fd)
+{
+       int rc = 0;
+  
+       if (shutdown(fd, 2) == -1)
+	 rc = errno;
+
+       return rc;
+}
+
 static int
 jthreadedListen(int fd, int l)
 {
@@ -196,11 +208,11 @@ jthreadedKill(int pid, int sig)
 }
 
 static int
-jthreadedBind(int fd, struct sockaddr *addr, int namelen)
+jthreadedBind(int fd, struct sockaddr *addr, socklen_t namelen)
 {
 	int rc = 0;
 
-	if (bind(fd, addr, (socklen_t)namelen) == -1) {
+	if (bind(fd, addr, namelen) == -1) {
 		rc = errno;
 	}
 	return (rc);
@@ -331,7 +343,7 @@ jthreadedSetSockOpt(int a, int b, int c, const void* d, int e)
 }
 
 static int
-jthreadedGetSockOpt(int a, int b, int c, void* d, int* e)
+jthreadedGetSockOpt(int a, int b, int c, void* d, socklen_t* e)
 {
 	int rc = 0;
 
@@ -342,7 +354,7 @@ jthreadedGetSockOpt(int a, int b, int c, void* d, int* e)
 }
 
 static int
-jthreadedGetSockName(int a, struct sockaddr* b, int* c)
+jthreadedGetSockName(int a, struct sockaddr* b, socklen_t* c)
 {
 	int rc = 0;
 
@@ -353,7 +365,7 @@ jthreadedGetSockName(int a, struct sockaddr* b, int* c)
 }
 
 static int
-jthreadedGetPeerName(int a, struct sockaddr* b, int* c)
+jthreadedGetPeerName(int a, struct sockaddr* b, socklen_t* c)
 {
 	int rc = 0;
 
@@ -420,7 +432,7 @@ jthreadedSelect(int a, fd_set* b, fd_set* c, fd_set* d,
  * Threaded socket connect.
  */
 static int
-jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout)
+jthreadedConnect(int fd, struct sockaddr* addr, socklen_t len, int timeout)
 {
 	int r;
 	jlong deadline = 0;
@@ -428,7 +440,7 @@ jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout)
 
 	SET_DEADLINE(deadline, timeout)
 	for (;;) {
-		r = connect(fd, addr, (socklen_t)len);
+		r = connect(fd, addr, len);
 		if (r == 0 || !(errno == EINPROGRESS 
 				|| errno == EINTR || errno == EISCONN)) {
 			break;	/* success or real error */
@@ -467,12 +479,12 @@ jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout)
  * Threaded socket accept.
  */
 static int
-jthreadedAccept(int fd, struct sockaddr* addr, int* len, 
+jthreadedAccept(int fd, struct sockaddr* addr, socklen_t* len, 
 		int timeout, int* out)
 {
 	/* absolute time at which time out is reached */
 	int r=-1, ret;	
-	ret = waitForTimeout(fd,timeout);
+	ret = waitForRW(fd,timeout);
 
 	/* If result is 0, we had a timeout. 
 	 * If it's not, let's try to accept.
@@ -581,9 +593,9 @@ jthreadedWrite(int fd, const void* buf, size_t len, ssize_t *out)
  */
 static int 
 jthreadedRecvfrom(int fd, void* buf, size_t len, int flags, 
-	struct sockaddr* from, int* fromlen, int timeout, ssize_t *out)
+	struct sockaddr* from, socklen_t* fromlen, int timeout, ssize_t *out)
 {
-	int r;
+	int r = 0;
 	jlong deadline = 0;
 	int poll_timeout;
 	int blocking = jthread_is_blocking(fd);
@@ -591,17 +603,20 @@ jthreadedRecvfrom(int fd, void* buf, size_t len, int flags,
 	jthread_set_blocking(fd, 0);
 	SET_DEADLINE(deadline, timeout)
 	for (;;) {
-		r = recvfrom(fd, buf, len, flags, from, fromlen);
-		if (r >= 0 || !(errno == EWOULDBLOCK || errno == EINTR 
-					|| errno == EAGAIN)) {
-			break;
-		}
-		IGNORE_EINTR(r)
-		poll_timeout = deadline - currentTime();
-		if (poll_timeout > 0 || timeout == NOTIMEOUT) {
-			waitForTimeout(fd, poll_timeout);
+		if (timeout != NOTIMEOUT) {
+		        poll_timeout = deadline - currentTime();
+			if (poll_timeout > 0)
+			  waitForTimeout(fd, poll_timeout);
+		} else {
+			waitForTimeout(fd, NOTIMEOUT);
 		}
 		BREAK_IF_LATE(deadline, timeout)
+
+		r = recvfrom(fd, buf, len, flags, from, fromlen);
+		if (r >= 0 || !(errno == EWOULDBLOCK || errno == EINTR
+			|| errno == EAGAIN)) {
+			break;
+		}
 	}
 	jthread_set_blocking(fd, blocking);
 	SET_RETURN_OUT(r, out, r)
@@ -897,6 +912,7 @@ SystemCallInterface Kaffe_SystemCallInterface = {
         jthreadedGetSockName, 
         jthreadedGetPeerName, 
         jthreadedClose,
+	jthreadedSocketShutdown,
         jthreadedGetHostByName,
         jthreadedGetHostByAddr,
         jthreadedSelect,	
