@@ -12,7 +12,7 @@
  * of this file.
  */
 
-#include <errno.h>
+#include "lerrno.h"
 #include <limits.h>
 
 #include "config.h"
@@ -21,6 +21,22 @@
 #include "config-setjmp.h"
 #include "config-io.h"
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
 #include "locks.h"
 #include "thread-impl.h"
 #include "debug.h"
@@ -258,9 +274,8 @@ tDumpList ( jthread_t cur, jthread_t list )
 
 	dprintf("%4d: %c%c%c %c%c%c   %p [tid: %4ld, java: %p]  "
 		"stack: [%p..%p..%p]\n",
-		i, a1, a2, a3,
-		stat_act[t->active], stat_susp[t->suspendState], stat_block[t->blockState],
-		t, t->tid, t->data.jlThread,
+		i, a1, a2, a3, stat_act[t->active], stat_susp[t->suspendState],
+		stat_block[t->blockState], t, (long)t->tid, t->data.jlThread,
 		t->stackMin, t->stackCur, t->stackMax);
   }
 }
@@ -302,14 +317,14 @@ tDump (void)
  */
 static
 void
-dump_signal_handler ( UNUSED int sig )
+dump_signal_handler (int sig UNUSED)
 {
   tDump();
 }
 
 #ifdef KAFFE_VMDEBUG
 static
-void* tWatchdogRun (void* p)
+void* tWatchdogRun (void* p UNUSED)
 {
   jthread_t t;
   int life;
@@ -331,7 +346,7 @@ void* tWatchdogRun (void* p)
 	if ( !life ) {
 	  DBG( JTHREAD, dprintf("deadlock\n"));
 	  tDump();
-	  ABORT();
+	  KAFFEVM_ABORT();
 	}
 
 	usleep( 5000);
@@ -400,7 +415,7 @@ tInitSignalHandlers (void)
   saResume.sa_flags = 0; /* Note that we do not want restart here. */
   saResume.sa_handler = resume_signal_handler;
   saResume.sa_mask = saSuspend.sa_mask;
-#ifndef KAFFE_BOEHM_GC
+#if !defined(KAFFE_BOEHM_GC) && !defined(KAFFE_BUGGY_NETBSD_SIGWAIT)
   sigaction( sigResume, &saResume, NULL);
 #endif
 
@@ -546,8 +561,8 @@ tSetupFirstNative(void)
  * alternative to scattered pthread_once() calls
  */
 void
-jthread_init(UNUSED int pre,
-        int maxpr, int UNUSED minpr,
+jthread_init(int pre UNUSED,
+        int maxpr, int minpr UNUSED,
 	Collector *thread_collector,
 	void (*destructor)(void *),
         void (*_onstop)(void) UNUSED,
@@ -579,7 +594,8 @@ jthread_init(UNUSED int pre,
 }
 
 jthread_t
-jthread_createfirst(size_t mainThreadStackSize, UNUSED unsigned int pri, void* jlThread)
+jthread_createfirst(size_t mainThreadStackSize, unsigned int pri UNUSED,
+		    void* jlThread)
 {
   jthread_t      nt;
   int            oldCancelType;
@@ -966,6 +982,8 @@ jthread_create ( unsigned int pri, void* func, int isDaemon, void* jlThread, siz
 	  case EPERM:
 	    DBG( JTHREAD, dprintf( "no permission to set scheduling\n"));
 	    break;
+	  default:
+	    break;
 	  }
 
 	  repsem_destroy( &nt->sem);
@@ -1101,7 +1119,7 @@ jthread_exit ( void )
 	    pthread_exit( NULL);
 
 	    /* we shouldn't get here, this is a last safeguard */
-	    EXIT(0);
+	    KAFFEVM_EXIT(0);
 	  }
 	}
 	unprotectThreadList(cur);
@@ -1128,7 +1146,7 @@ jthread_exit ( void )
 	 * real threads to terminate. The main thread gets the control back
 	 * after that.
 	 */
-	repsem_wait( &cur->sem);
+	while (repsem_wait( &cur->sem) != 0);
   }
   else if (cur != firstThread) {
 	/* flag that we soon will get a new cache entry (would be annoying to
@@ -1136,6 +1154,7 @@ jthread_exit ( void )
 	pendingExits++;
 	unprotectThreadList(cur);
   }
+  else unprotectThreadList(cur);
 }
 
 
@@ -1256,7 +1275,12 @@ void KaffePThread_WaitForResume(int releaseMutex, unsigned int state)
 
   /* freeze until we get a subsequent sigResume */
   while( cur->suspendState == SS_SUSPENDED )
-    sigwait( &suspendSet, &s);
+    {
+      sigwait( &suspendSet, &s);
+      /* Post something even if it is not the right thread. */
+      if (cur->suspendState == SS_SUSPENDED)
+        repsem_post(&critSem);
+    }
   
   DBG( JTHREADDETAIL, dprintf("sigwait return: %p\n", cur));
 
@@ -1289,7 +1313,7 @@ void KaffePThread_WaitForResume(int releaseMutex, unsigned int state)
  * SA_ONSTACK / signalstack() in effect
  */
 void
-suspend_signal_handler ( UNUSED int sig )
+suspend_signal_handler (int sig UNUSED)
 {
   volatile jthread_t   cur = jthread_current();
 
@@ -1331,7 +1355,7 @@ void KaffePThread_AckAndWaitForResume(volatile jthread_t cur, unsigned int state
  * call (i.e. to unblock a preceeding sigwait).
  */
 void
-resume_signal_handler ( UNUSED int sig )
+resume_signal_handler ( int sig UNUSED )
 {
   /* we don't do anything, here - all the action is in the suspend handler */
 }
@@ -1359,7 +1383,8 @@ jthread_suspendall (void)
   protectThreadList(cur);
 
   DBG( JTHREAD, dprintf("enter crit section[%d] from: %p [tid:%4ld, java:%p)\n",
-			critSection, cur, cur->tid, cur->data.jlThread));
+			critSection, cur, (unsigned long)cur->tid,
+			cur->data.jlThread));
 
   if ( ++critSection == 1 ){
 
@@ -1395,7 +1420,7 @@ jthread_suspendall (void)
 		    if ((status = pthread_kill( t->tid, sigSuspend)) != 0)
 		      {
 			dprintf("Internal error: error sending SUSPEND signal to %p: %d (%s)\n", t, status, strerror(status));
-			ABORT();
+			KAFFEVM_ABORT();
 		      }
 		    else
 		      {
@@ -1477,13 +1502,17 @@ jthread_unsuspendall (void)
 	      if ((t->blockState & (BS_SYSCALL|BS_CV|BS_CV_TO|BS_MUTEX)) == 0)
 		{
 		  DBG (JTHREADDETAIL, dprintf("  sending sigResume\n"));
-		  status = pthread_kill( t->tid, sigResume);
-		  if ( status )
+		  do
 		    {
-		      DBG( JTHREAD, dprintf("error sending RESUME signal to %p: %d\n", t, status));
-		    }		  
-		  /* ack wait workaround, see jthread_suspendall remarks */
-		  repsem_wait( &critSem);
+		      status = pthread_kill( t->tid, sigResume);
+		      if ( status )
+			{
+			  DBG( JTHREAD, dprintf("error sending RESUME signal to %p: %d\n", t, status));
+			}		  
+		      /* ack wait workaround, see jthread_suspendall remarks */
+		      repsem_wait( &critSem);
+		    }
+		  while (t->suspendState == SS_PENDING_RESUME);
 		}
 	      else
 		{
@@ -1680,7 +1709,7 @@ DBG(JTHREADDETAIL, dprintf(" no\n"); );
  *
  * @param dummy unused pointer
  */
-void jthread_spinon(UNUSED void *dummy)
+void jthread_spinon(void *dummy UNUSED)
 {
   pthread_mutex_lock(&systemMutex);
 }
@@ -1691,7 +1720,7 @@ void jthread_spinon(UNUSED void *dummy)
  *
  * @param dummy unused pointer
  */
-void jthread_spinoff(UNUSED void *dummy)
+void jthread_spinoff(void *dummy UNUSED)
 {
   pthread_mutex_unlock(&systemMutex);
 }
@@ -1774,7 +1803,7 @@ void jthread_enable_stop(void)
  * 
  * @param tid the thread to stop.
  */
-void jthread_stop(UNUSED jthread_t tid)
+void jthread_stop(jthread_t tid UNUSED)
 {
 }
 
@@ -1783,7 +1812,7 @@ void jthread_stop(UNUSED jthread_t tid)
  *
  * @param tid the thread whose info is to be dumped.
  */
-void jthread_dumpthreadinfo(UNUSED jthread_t tid)
+void jthread_dumpthreadinfo(jthread_t tid UNUSED)
 {
 }
 
@@ -1834,23 +1863,68 @@ void* jthread_stacklimit(void)
 #endif
 }
 
-void jthread_suspend(UNUSED jthread_t jt, UNUSED void *suspender)
+int jthread_on_condvar(jthread_t jt)
+{
+	return (jt->blockState & (BS_CV|BS_CV_TO)) != 0;
+}
+
+int jthread_on_mutex(jthread_t jt)
+{
+	return (jt->blockState & (BS_MUTEX)) != 0;
+}
+
+int jthread_get_status(jthread_t jt)
+{
+  if (jt->status == THREAD_KILL)
+    return THREAD_DEAD;
+
+  return jt->status;
+}
+
+int jthread_has_run(jthread_t jt UNUSED)
+{
+  return 1;
+}
+
+void jthread_clear_run(jthread_t jt UNUSED)
+{
+}
+
+void jthread_suspend(jthread_t jt UNUSED, void *suspender UNUSED)
 {
 	/* TODO */
 }
 
-void jthread_resume(UNUSED jthread_t jt, UNUSED void *suspender)
+void jthread_resume(jthread_t jt UNUSED, void *suspender UNUSED)
 {
 	/* TODO */
 }
 
-jthread_t jthread_from_data(UNUSED threadData *td, UNUSED void *suspender)
+jthread_t jthread_from_data(threadData *td, void *suspender UNUSED)
 {
-	/* TODO */
-	return NULL;
+  jthread_t cur = jthread_current();
+  jthread_t iterator;
+
+  protectThreadList(cur);
+  iterator = activeThreads;
+  while (iterator != NULL)
+    {
+      if (td == &iterator->data)
+	{
+	  unprotectThreadList(cur);
+	  /* Thread handles are garbage collected so the stack is protecting
+	   * us.
+	   */
+	  return iterator;
+	}
+      iterator = iterator->next;
+    }
+
+  unprotectThreadList(cur);
+  return NULL;
 }
 
-jlong jthread_get_usage(UNUSED jthread_t jt)
+jlong jthread_get_usage(jthread_t jt UNUSED)
 {
 	/* TODO */
 	return 0;
