@@ -50,12 +50,12 @@ Hjava_lang_Thread* getCurrentThread(void);
 Hjava_lang_VMThread* getCurrentVMThread(void);
 Hjava_lang_Thread* createDaemon(void*, const char*, void *arg, int,
 				size_t, struct _errorInfo *);
-void	attachFakedThreadInstance (const char *name, int isDaemon);
+void	KaffeVM_attachFakedThreadInstance (const char *name, int isDaemon);
 
 extern  Hjava_lang_Class* ThreadClass;
 struct  _Collector;
 
-extern  void KaffeVM_setDeadlockDetection(jbool detection);
+extern  void KaffeVM_setDeadlockDetection(jboolean detection);
 
 #define THREAD_NATIVE()         ((void*)KTHREAD(current)())
 
