@@ -1,10 +1,13 @@
 /*
  * utf8const.c
- *
  * Handle UTF-8 constant strings. These are intern'ed into a hash table.
  *
- * Copyright (c) 1998, 2004
+ * Copyright (c) 1998
  *	Transvirtual Technologies, Inc.  All rights reserved.
+ *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *  
  */
 
 #include "config.h"
@@ -12,7 +15,7 @@
 #include "config-mem.h"
 #include "config-io.h"
 #include "classMethod.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "kaffe/jmalloc.h"
 #include "constants.h"
 #include "object.h"
@@ -33,6 +36,7 @@
 #define hashAdd(t, x)		(x)
 #define hashFind(t, x)		NULL
 #define hashRemove(t, x)	(void)NULL
+#define initStaticLock(a)	do { } while (0)
 #endif
 
 /* Internal variables */
@@ -95,8 +99,6 @@ utf8ConstNew(const char *s, int slen)
 	int32 hash;
 	Utf8Const *fake;
 	char buf[200];
-#if !defined(KAFFEH)
-#endif
 
 	/* Automatic length finder */
 	if (slen < 0) {
@@ -178,13 +180,13 @@ utf8ConstNew(const char *s, int slen)
 	 *		   add additional ref to other utf8
 	 */
 
-	if (temp != 0 && temp != utf8) {
+	if (temp != NULL && temp != utf8) {
 		temp->nrefs++;
 	}
 
 	unlockUTF();
 
-	if (temp == 0 || temp != utf8) {
+	if (temp == NULL || temp != utf8) {
 		gc_free(utf8);
 	}
 
@@ -198,8 +200,6 @@ utf8ConstNew(const char *s, int slen)
 void
 utf8ConstAddRef(Utf8Const *utf8)
 {
-#if !defined(KAFFEH)
-#endif
 	lockUTF();
 	assert(utf8->nrefs >= 1);
 	utf8->nrefs++;
@@ -212,8 +212,6 @@ utf8ConstAddRef(Utf8Const *utf8)
 void
 utf8ConstRelease(Utf8Const *utf8)
 {
-#if !defined(KAFFEH)
-#endif
 	/* NB: we ignore zero utf8s here in order to not having to do it at
 	 * the call sites, such as when destroying half-processed class 
 	 * objects because of error conditions.
@@ -369,9 +367,7 @@ utf8ConstInit(void)
 {
 	DBG(INIT, dprintf("utf8ConstInit()\n"); );
 
-#if !defined(KAFFEH)
 	initStaticLock(&utf8Lock);
-#endif
 
 	lockUTF();
 	hashTable = hashInit(utf8ConstHashValueInternal,
@@ -398,7 +394,7 @@ utf8ConstEqual(Utf8Const* a, Utf8Const* b)
 	/* If they're different pointers, double check that they're different strings... */
 	if ((a != b) && (a->hash == b->hash))
 	{
-		assert(strcmp(a->data,b->data));
+		assert(strcmp(a->data,b->data) != 0);
 	}
 #endif
 	/* Since we intern all UTF-8 constants, we can do this: */
