@@ -13,13 +13,12 @@
 #include "config.h"
 #include "config-std.h"
 #include "gtypes.h"
-#include "kaffe_management_JIT.h"
+#include "org_kaffe_management_JIT.h"
 #include "classMethod.h"
 #include "lookup.h"
 #include "support.h"
 #include "debug.h"
 #include "stringSupport.h"
-#include "methodCache.h"
 
 /* XXX Temporary for now until we define interface header file that
  * declares "translate"
@@ -39,7 +38,7 @@
  * Translate a method given class, method name, and signature
  */
 void
-Java_kaffe_management_JIT_translateMethod(JNIEnv *env UNUSED, jclass _vmclass UNUSED, 
+Java_org_kaffe_management_JIT_translateMethod(JNIEnv *env UNUSED, jclass _vmclass UNUSED, 
 	jstring _cname, jstring _mname, 
 	jstring _signature, jobject loader)	
 	/* JNI? what's that? */
@@ -90,7 +89,7 @@ Java_kaffe_management_JIT_translateMethod(JNIEnv *env UNUSED, jclass _vmclass UN
  * Flush the dcache 
  */
 void
-Java_kaffe_management_JIT_flushCache(JNIEnv *env UNUSED, jclass clazz UNUSED)
+Java_org_kaffe_management_JIT_flushCache(JNIEnv *env UNUSED, jclass clazz UNUSED)
 {
 #if defined(TRANSLATOR) 
 #if defined(FULL_CACHE_FLUSH)
