@@ -22,14 +22,6 @@
 #include "exception.h"
 #include "baseClasses.h"
 #include "java_lang_reflect_Method.h"
-#include "java_lang_Boolean.h"
-#include "java_lang_Byte.h"
-#include "java_lang_Character.h"
-#include "java_lang_Short.h"
-#include "java_lang_Integer.h"
-#include "java_lang_Long.h"
-#include "java_lang_Float.h"
-#include "java_lang_Double.h"
 #include <native.h>
 #include "jni.h"
 #include "defs.h"
@@ -170,8 +162,8 @@ Java_java_lang_reflect_Method_invoke0(JNIEnv* env, jobject _this, jobject _obj,
 	rettype = *METHOD_RET_TYPE(meth);
 
 	for (i = len - 1; i >= 0; i--) {
-		arg = (*env)->GetObjectArrayElement(env, argobj, (unsigned int)i);
-		argc = (*env)->GetObjectArrayElement(env, paramtypes, (unsigned int)i);
+		arg = (*env)->GetObjectArrayElement(env, argobj, i);
+		argc = (*env)->GetObjectArrayElement(env, paramtypes, i);
 		if (!CLASS_IS_PRIMITIVE((Hjava_lang_Class*)argc)) {
 			args[i].l = arg;
 		}
@@ -200,6 +192,8 @@ Java_java_lang_reflect_Method_invoke0(JNIEnv* env, jobject _this, jobject _obj,
 		case 'J':
 			args[i].j = (*env)->GetLongField(env, arg, Jvalue);
 			break;
+		default:
+			break;
 		}
 	}
 
@@ -225,7 +219,7 @@ Java_java_lang_reflect_Method_invoke0(JNIEnv* env, jobject _this, jobject _obj,
 		case '[': ret.l = CallStaticTypeMethodA(Object); break;
 #undef CallStaticTypeMethodA
 		default:
-			ABORT();
+			KAFFEVM_ABORT();
 		}
 	}
 	else if (METHOD_IS_CONSTRUCTOR(meth)) { /* a constructor */
@@ -264,7 +258,7 @@ Java_java_lang_reflect_Method_invoke0(JNIEnv* env, jobject _this, jobject _obj,
 		case '[': ret.l = CallTypeMethodA(Object); break;
 #undef CallTypeMethodA
 		default:
-			ABORT();
+			KAFFEVM_ABORT();
 		}
 	}
 
@@ -311,6 +305,8 @@ Java_java_lang_reflect_Method_invoke0(JNIEnv* env, jobject _this, jobject _obj,
 	case 'L':
 	case '[':
 		return (ret.l);
+	default:
+		break;
 	}
 
 	return (NULL);
