@@ -33,7 +33,7 @@
  * Return class object for this object.
  */
 struct Hjava_lang_Class*
-java_lang_Object_getClass(struct Hjava_lang_Object* o)
+java_lang_VMObject_getClass(struct Hjava_lang_Object* o)
 {
   return (OBJECT_CLASS(o));
 }
@@ -84,7 +84,7 @@ java_lang_VMObject_clone(struct Hjava_lang_Cloneable* c)
       /* Clone an array */
       obj = newArray(CLASS_ELEMENT_TYPE(class), ARRAY_SIZE(o));
       memcpy(ARRAY_DATA(obj), ARRAY_DATA(o),
-	     ARRAY_SIZE(o) * TYPE_SIZE(CLASS_ELEMENT_TYPE(class)));
+	     (size_t) ARRAY_SIZE(o) * TYPE_SIZE(CLASS_ELEMENT_TYPE(class)));
     }
   return (obj);
 }
@@ -112,7 +112,7 @@ DBG(VMTHREAD, dprintf ("%p (%p) waiting for %p, %lli\n",
       JVMPI_Event ev;
       
       ev.event_type = JVMPI_EVENT_MONITOR_WAIT;
-      if( o == getCurrentThread()->vmThread )
+      if( o == (struct Hjava_lang_Object*) getCurrentThread()->vmThread )
 	{
 	  ev.u.monitor_wait.object = NULL;
 	}
@@ -126,12 +126,12 @@ DBG(VMTHREAD, dprintf ("%p (%p) waiting for %p, %lli\n",
 #endif
   waitCond(o, timeout);
 #if defined(ENABLE_JVMPI)
-  if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_MONITOR_WAIT) )
+  if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_MONITOR_WAITED) )
     {
       JVMPI_Event ev;
       
-      ev.event_type = JVMPI_EVENT_MONITOR_WAIT;
-      if( o == getCurrentThread()->vmThread )
+      ev.event_type = JVMPI_EVENT_MONITOR_WAITED;
+      if( o == (struct Hjava_lang_Object*) getCurrentThread()->vmThread )
 	{
 	  ev.u.monitor_wait.object = NULL;
 	}
