@@ -128,7 +128,7 @@ gnu_java_net_PlainDatagramSocketImpl_bind(struct Hgnu_java_net_PlainDatagramSock
 {
 	int r;
 	KaffeSocketAddr addr;
-	int alen;
+	socklen_t alen;
 	const int fd = unhand(obj)->native_fd;
 
 	memset(&addr, 0, sizeof(addr));
@@ -209,7 +209,7 @@ gnu_java_net_PlainDatagramSocketImpl_send0(struct Hgnu_java_net_PlainDatagramSoc
 	int rc;
 	ssize_t bsent;
 	KaffeSocketAddr addr;
-	int alen;
+	int alen = 0;
 	
 DBG(NATIVENET,
 	dprintf("datagram_send(%p, %p [%d bytes])\n",
@@ -259,7 +259,7 @@ DBG(NATIVENET,
 	}
 
 	rc = KSENDTO(unhand(obj)->native_fd,
-		unhand_array(unhand(pkt)->buffer)->body, unhand(pkt)->length,
+		unhand_array(unhand(pkt)->buffer)->body, (size_t) unhand(pkt)->length,
 		0, (struct sockaddr *)&addr, alen, &bsent);
 
 DBG(NATIVENET,
@@ -277,7 +277,7 @@ gnu_java_net_PlainDatagramSocketImpl_peek(struct Hgnu_java_net_PlainDatagramSock
 	ssize_t r;
 	int rc;
 	KaffeSocketAddr saddr;
-	int alen = sizeof(saddr);
+	socklen_t alen = sizeof(saddr);
 
 	rc = KRECVFROM(unhand(obj)->native_fd,
 		NULL, 0, MSG_PEEK, (struct sockaddr*)&saddr,
@@ -306,7 +306,7 @@ gnu_java_net_PlainDatagramSocketImpl_receive0(struct Hgnu_java_net_PlainDatagram
 	ssize_t r;
 	int rc;
 	KaffeSocketAddr addr;
-	int alen = sizeof(addr);
+	socklen_t alen = sizeof(addr);
 	HArrayOfByte *array_address;
 	int to_read, offset;
 
@@ -421,7 +421,8 @@ void
 gnu_java_net_PlainDatagramSocketImpl_socketSetOption(struct Hgnu_java_net_PlainDatagramSocketImpl* obj, jint opt, struct Hjava_lang_Object* arg)
 {
 	struct Hjava_net_InetAddress* addrp;
-	int k, v, r;
+	int v, r;
+	unsigned int k;
 
 	/* Do easy cases */
 	for (k = 0; k < sizeof(socketOptions) / sizeof(*socketOptions); k++) {
@@ -480,12 +481,13 @@ gnu_java_net_PlainDatagramSocketImpl_socketSetOption(struct Hgnu_java_net_PlainD
 jint
 gnu_java_net_PlainDatagramSocketImpl_socketGetOption(struct Hgnu_java_net_PlainDatagramSocketImpl* obj, jint opt)
 {
-	int k, r, v;
-	int vsize = sizeof(v);
+	int r = 0, v;
+	unsigned int k;
+	socklen_t vsize = sizeof(v);
 	struct sockaddr_in addr;
-	int alen = sizeof(addr);
+	socklen_t alen = sizeof(addr);
 	struct in_addr ia;
-	int ia_len = sizeof(ia);
+	socklen_t ia_len = sizeof(ia);
 
 	/* Do easy cases */
 	for (k = 0; k < sizeof(socketOptions) / sizeof(*socketOptions); k++) {
@@ -617,7 +619,7 @@ gnu_java_net_PlainDatagramSocketImpl_joinGroup(struct Hgnu_java_net_PlainDatagra
 #if defined(IP_ADD_MEMBERSHIP)
 	struct Hjava_net_InetSocketAddress *jisa;
 	struct ip_mreq ipm;
-	int r;
+	int r = 0;
 
 	jisa = (struct Hjava_net_InetSocketAddress *)jsa;
 
@@ -714,7 +716,7 @@ gnu_java_net_PlainDatagramSocketImpl_leaveGroup(struct Hgnu_java_net_PlainDatagr
  * set multicast-TTL
  */
 void
-gnu_java_net_PlainDatagramSocketImpl_setTTL(struct Hgnu_java_net_PlainDatagramSocketImpl* obj, jbool ttl)
+gnu_java_net_PlainDatagramSocketImpl_setTTL(struct Hgnu_java_net_PlainDatagramSocketImpl* obj, jbyte ttl)
 {
 
 #if defined(IP_MULTICAST_TTL)
@@ -742,7 +744,7 @@ gnu_java_net_PlainDatagramSocketImpl_getTTL(struct Hgnu_java_net_PlainDatagramSo
 
 #if defined(IP_MULTICAST_TTL)
 	unsigned char v;
-	int s = 1;
+	socklen_t s = 1;
 	int r;
 
 	r = KGETSOCKOPT(unhand(obj)->native_fd,
