@@ -15,8 +15,6 @@
 #include "java_util_zip_Adler32.h"
 #include "gtypes.h"
 
-#if defined(HAVE_LIBZ) && defined(HAVE_ZLIB_H)
-
 #include <zlib.h>
 
 void
@@ -37,25 +35,3 @@ java_util_zip_Adler32_update1(struct Hjava_util_zip_Adler32* this, jint val)
 	unhand(this)->adler = adler32((uint32)unhand(this)->adler, &b, sizeof(b));
 }
 
-#else
-
-static void 
-supportDisabled (void)
-{
-	SignalError ("org.kaffe.util.SupportDisabled",
-		     "libz was not found by Kaffe configure script");
-}
-
-void
-java_util_zip_Adler32_update(struct Hjava_util_zip_Adler32* this, HArrayOfByte* buf, jint from, jint len)
-{
-	supportDisabled();
-}
-
-void
-java_util_zip_Adler32_update1(struct Hjava_util_zip_Adler32* this, jint val)
-{
-	supportDisabled();
-}
-
-#endif
