@@ -1,3 +1,19 @@
+/**
+ * Component - abstract root of all widgets
+ *
+ * Copyright (c) 1998
+ *      Transvirtual Technologies, Inc.  All rights reserved.
+ * Copyright (c) 2006
+ *      Kaffe.org developers. See ChangeLog for details.
+ *
+ * See the file "license.terms" for information on usage and redistribution 
+ * of this file. 
+ *
+ * original code P.C.Mehlitz
+ * some code taken or adapted from Classpath
+ */
+ 
+ 
 package java.awt;
 
 import gnu.classpath.Pointer;
@@ -13,6 +29,7 @@ import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
+import java.awt.event.MouseWheelListener;
 import java.awt.event.MouseMotionListener;
 import java.awt.event.PaintEvent;
 import java.awt.event.TextEvent;
@@ -25,20 +42,13 @@ import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.Serializable;
 import java.util.Locale;
+import java.awt.dnd.DropTarget;
 
 import org.kaffe.awt.DoNothingPeer;
 
-/**
- * Component - abstract root of all widgets
- *
- * Copyright (c) 1998
- *    Transvirtual Technologies, Inc.  All rights reserved.
- *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
- *
- * @author P.C.Mehlitz
- */
+
+ 
+ 
 abstract public class Component
   extends Object
   implements ImageObserver, MenuContainer, Serializable
@@ -417,19 +427,41 @@ public Color getBackground () {
 */
 }
 
-/**
- * @deprecated, use getBounds()
- */
 public Rectangle getBounds () {
 	return bounds();
 }
 
+// TODO this is a tentative implementation
+public Rectangle getBounds (Rectangle rv) {
+	if (rv == null)
+		return bounds();	
+	rv.x = x;
+	rv.y = y;
+	rv.width= width;
+	rv.height = height;
+	return rv;
+}
+
+
 ClassProperties getClassProperties () {
 	// direct Component / Container derived classes can't use old events
 	// (they had no protected ctor in 1.0.2)
 	return ClassAnalyzer.analyzeProcessEvent( getClass(), false);
 }
 
+// TODO this is only a stub
+protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
+}
+
+// TODO this is only a stub
+protected void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
+}
+
+// TODO this is only a stub
+protected void firePropertyChange(String propertyName, int oldValue, int newValue) {
+}
+
+
 public ColorModel getColorModel() {
   	return Toolkit.getDefaultToolkit().getColorModel();
 }
@@ -502,6 +534,11 @@ public Dimension getMaximumSize() {
 	return Toolkit.singleton.getScreenSize();
 }
 
+// Subclasses that guarantee to always completely paint their contents should override this method and return true. All of the "heavyweight" AWT components are opaque.
+public boolean isOpaque() {
+	return false;
+}
+
 public Dimension getMinimumSize() {
 	return minimumSize();
 }
@@ -540,6 +577,21 @@ public Dimension getSize () {
 	return size();
 }
 
+/**
+ * this is a stub only for now
+ */
+public void setDropTarget(DropTarget dt) {
+// TODO this is only a stub	
+}
+
+/**
+ * this is a stub only for now
+ */
+public DropTarget getDropTarget() {
+// TODO this is only a stub
+	return null;
+}
+
 public Toolkit getToolkit () {
 	return Toolkit.singleton;
 }
@@ -721,6 +773,29 @@ public boolean isFocusTraversable() {
 	        ((eventMask & AWTEvent.DISABLED_MASK) == 0));
 }
 
+// TODO this is was copied from isFocusTraversable, may be wrong.
+public boolean isFocusable() {
+	return (((flags & (IS_SHOWING|IS_NATIVE_LIKE)) == (IS_SHOWING|IS_NATIVE_LIKE)) && 
+	        ((eventMask & AWTEvent.DISABLED_MASK) == 0));
+}
+
+// TODO this is a stub only
+public void setFocusable(boolean focusable) {
+}
+
+// TODO this is a stub only
+public boolean isFocusOwner() {
+	return true;
+}
+
+/**
+ * @deprecated
+ */
+public boolean hasFocus() {
+	return isFocusOwner();
+}
+
+
 public boolean isShowing () {
 	// compare the costs of this with the standard upward iteration
  return  ((flags & (IS_PARENT_SHOWING | IS_VISIBLE | IS_ADD_NOTIFIED)) ==
@@ -1815,6 +1890,19 @@ public void update ( Graphics g ) {
 	paint( g);
 }
 
+// TODO this is only a stub
+public ComponentOrientation getComponentOrientation() {
+	return ComponentOrientation.LEFT_TO_RIGHT;
+}
+
+// TODO this is only a stub
+public void applyComponentOrientation(ComponentOrientation orientation) {
+}
+
+// TODO this is only a stub
+public void addMouseWheelListener(MouseWheelListener l) {
+}
+
 synchronized void updateLinkedGraphics () {
 	GraphicsLink li, last, next;
 
