@@ -20,7 +20,7 @@
  * Copyright (c) 1998
  *      Transvirtual Technologies, Inc.  All rights reserved.
  *
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005
  * 	The Kaffe.org's developers. See ChangeLog for details.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -32,12 +32,12 @@
 package java.awt;
 
 import java.awt.image.ImageObserver;
-import kaffe.util.Ptr;
+import gnu.classpath.Pointer;
 
 class NativeGraphics
   extends Graphics
 {
-	Ptr nativeData;
+	Pointer nativeData;
 	int xOffset;
 	int yOffset;
 	Color fgClr;
@@ -173,6 +173,11 @@ public void dispose () {
 		bgClr = null;
 	}
 
+	if ( nativeData != null ) {
+		Toolkit.graFreeGraphics( nativeData);
+		nativeData = null;
+	}
+
 	synchronized ( lock ) {
 		next = cache;
 		cache = this;
@@ -528,7 +533,7 @@ public FontMetrics getFontMetrics ( Font fnt ) {
 	return FontMetrics.getFontMetrics( fnt);
 }
 
-static NativeGraphics getGraphics ( Object target, Ptr tgtData, int tgtType,
+static NativeGraphics getGraphics ( Object target, Pointer tgtData, int tgtType,
                   int xOffset, int yOffset,
 									int xClip, int yClip, int wClip, int hClip,
 									Color fg, Color bg, Font fnt, boolean blank ) {
@@ -681,7 +686,7 @@ public void setFont ( Font newFnt ) {
 	}
 }
 
-void setGraphics ( Ptr tgtData, int tgtType, int xOffset, int yOffset,
+void setGraphics ( Pointer tgtData, int tgtType, int xOffset, int yOffset,
 		               int xClip, int yClip, int wClip, int hClip,
 		               Color fg, Color bg, Font fnt, boolean blank ) {
 
