@@ -33,7 +33,7 @@ public class ProcessClassStop
 	Thread wd = new Thread() {
 	    public void run() {
 		try {
-		    Thread.sleep(3000);
+		    Thread.sleep(10000);
 		    System.out.println("sorry, you timed out");
 		    System.exit(-1);
 		} catch (Exception e) {
@@ -105,8 +105,9 @@ public class ProcessClassStop
 /* Expected Output:
 tan java.lang.NoClassDefFoundError: Base
 java.lang.NoClassDefFoundError: Base
-   at java.lang.Class.getConstructor0 (Class.java)
-   at java.lang.Class.getDeclaredConstructor (Class.java:174)
-   at java.lang.Class.newInstance (Class.java:419)
+   at java.lang.reflect.Method.invoke0 (Method.java:native)
+   at java.lang.reflect.Method.invoke (Method.java:255)
+   at java.lang.reflect.Constructor.newInstance (Constructor.java:90)
+   at java.lang.Class.newInstance (Class.java:1143)
    at ProcessClassStop.main (ProcessClassStop.java:74)
 */
